/*	@(#)tmpl.h   MgnAlt 1.4 (MIT/CSR) 6/5/90

        Function:       Structure of Magellan radar ambiguity function
        System:         Magellan Altimetry Error Analysis
        Program:        mgmalt
        Module:         tmpl.h
        Revision:       1.4
        Date:           6/5/90
        Author:         pgf@space.mit.edu
        Created:        Tue Sep 13 15:01:04 EDT 1988
        Checker:
        Checked:
        Control:        SCCS
*/

#define	AMBIG_VERSION	1		/* Version number of ambig funct */
#define	PARTL_MAGIC	0x22ccaa44	/* Magic partials file number */
#define	PARTL_VERSION	1		/* Version number of partials file */

	/*
	**	Mean orbit parameters, etc.
	*/

#define SMA		10190.3		/* Semi-major axis */
#define ECC		0.38167		/* Eccentricity */
#define PERIOD		(3.15*3600)	/* Orbit period */
#define	INCL		RAD(86.0)	/* Inclination */
#define	ASCAN		RAD(-61.183)	/* RA of ascending node */
#define	ARGPER		RAD(170.0)	/* Argument of periapsis */
#define RADIUS		6051.5		/* Venus planetary radius (km) */

	/*
	**	Miscellaneous
	*/

#define	VELC		0.2997925e6	/* Velocity of light (km/sec) */
#define	PI		3.141592654	/* pi */
#define	KBOLTZ		(1.38e-23)	/* Boltzmann constant */
#define	RHO		0.13		/* Average reflectivity */
#define	RAD(x)		((x)*M_PI/180)	/* convert degrees to radians */

	/*
	**	Characteristics of the radar
	*/

#define	FCAR		2385.0e6	/* Carrier frequency (hz) */
#define	BANDWIDTH	(2.26e+6)	/* inverse baud (hz) */
#define	LAMBDA		(VELC/FCAR)	/* Carrier wavelength (km) */
#define	CODEREP		15		/* pn repeat count */
#define	REPLEN		151		/* code repeat length */
#define	POWER		300.0		/* Peak power (W) */
#define	DBGAIN		18.0		/* One way system gain (dB) */
#define	SYSTEMP		1200.0		/* "System" temperature on planet (K) */
#define	NOISE_BANDWIDTH	(1.365302e6)	/* Noise bandwidth of baseband filt */
#define	BEAM10		(PI/36.0)	/* 5 degree halfwidth halfpower */
#define	BEAM30		(PI/12.0)	/* 15 degree halfwidth halfpower */

	/*
	**	Characteristics of the ambiguity function
	*/

#define	COHERENT_GAIN	(50.39)		/* Time compression gain factor */
#define	INCOHERENT_GAIN	(15.0)		/* Frequency resolution gain factor */
#define	AMBIG_NORM	(0.275972)	/* Ambiguity function peak value */

	/*
	**	Ambiguity function header
	*/

				/* Begin ambig.h */
typedef	struct	{		/* tmpl.h 1.4 6/5/90 */
	int	ah_version;	/* Version number */
	int	ah_delrep;	/* Delay sub-resolution */
	int	ah_doprep;	/* Doppler sub-resolution */
	int	ah_ndel0;	/* delay origin */
	int	ah_ndel;	/* number of delay cells */
	int	ah_ndop;	/* number of doppler cells */
} AMBIG;
				/* End ambig.h */

	/*
	**	Characteristics of template file
	*/

#define	NDEL0	(-150)		/* Starting delay */
#define	NDEL1	151		/* Ending delay */
#define	NDELS	(NDEL1-NDEL0+1)	/* Number of delays */

	/*
	**	Partials file header record
	*/

				/* Begin tmpl.h */
typedef	struct	{		/* tmpl.h 1.4 6/5/90 */
	int	pt_magic;	/* Partials file magic number */
	int	pt_version;	/* Partials software version */
	float	pt_alt;		/* S/C altitude */
	float	pt_cpar;	/* Hagfors parameter */
	float	pt_ddop;	/* Doppler shift (Hz) */
	float	pt_excdl;	/* Excess delay */
	float	pt_deldop;	/* Doppler resolution */
	int	pt_ndop;	/* Frequency shift index */
	int	pt_ndel0;	/* Starting delay */
	int	pt_ndel1;	/* Ending delay */
} PARTL;
				/* End tmpl.h */

	/*
	**	Partials file data
	*/

typedef	struct	{
	float	pt_value[NDELS];	/* Template value */
	float	pt_dv_ddel[NDELS];	/* Partial wrt. delay */
	float	pt_dv_ddop[NDELS];	/* Partial wrt. frequency */
	float	pt_dv_dcpar[NDELS];	/* Partial wrt. C parameter */
	float	pt_dv_doff[NDELS];	/* Parital wrt. antenna offset */
} PDATA;

