
/*
 * $Log: alttile.c,v $
 * Revision 14.0  1998/07/31 19:46:47  root
 * \#makecheckin
 *
 * Revision 13.0  1998/07/31 19:46:30  root
 * \#makecheckin
 *
 * Revision 12.0  1998/07/31 09:58:26  root
 * \#makecheckin
 *
 * Revision 11.0  1998/05/02 23:03:44  root
 * \#makecheckin
 *
 * Revision 10.2  1998/04/24 20:58:36  root
 * \#makecheckin
 *
 * Revision 10.1  1997/10/22 18:35:49  root
 * \#makecheckin
 *
 * Revision 10.0  1997/10/22 18:33:55  root
 * \#makecheckin
 *
 * Revision 6.0  1997/07/04  00:32:42  root
 * \#makecheckin
 *
 * Revision 5.1  1997/07/04 00:31:02  root
 * \#makecheckin
 *
 * Revision 5.0  1997/07/03 15:05:48  root
 * \#makecheckin
 *
 * Revision 4.0  1997/07/01 11:58:59  root
 * \#makecheckin
 *
 * Revision 3.0  1997/06/23 12:51:50  root
 * \#makecheckin
 *
 * Revision 2.0  1997/06/23 09:04:44  root
 * \#makecheckin
 *
 * Revision 1.13  1997/06/23 08:53:04  root
 * \#makecheckin
 *
 * Revision 1.12  1997/06/18 22:11:10  root
 * \#makecheckin
 *
 * Revision 1.11  1997/06/18 15:15:32  root
 * \#makecheckin
 *
 * Revision 1.10  1997/06/15 23:47:49  root
 * \#makecheckin
 *
 */

static char rcsid[] = "$Id: alttile.c,v 14.0 1998/07/31 19:46:47 root Exp root $";

#include "mgn.h"

/* Maximum number of feet to check in an orbit */
#define NFEET 2000


int alttileserial = 0;

static IOF
contiguous_ixy_to_iof (Alttile t, IXY ixy)
{
  IOF ret;

  ret.orbit = t->pos.orbit + ixy.x;
  ret.foot = t->pos.foot + ixy.y;

  return (ret);

}

static IXY
contiguous_iof_to_ixy (Alttile t, IOF iof)
{
  IXY ret;

  ret.x = iof.orbit - t->pos.orbit;
  ret.y = iof.foot - t->pos.foot;

  return (ret);

}



static struct alttile_funcs contiguous_funcs =
{contiguous_ixy_to_iof,
 contiguous_iof_to_ixy};


static IOF
one_orbit_ixy_to_iof (Alttile t, IXY ixy)
{
  IOF ret;

  ret.orbit = t->pos.orbit + ixy.x;

  while (orbit_to_cycle (ret.orbit) != t->cycle)
    {
      ret.orbit -= ORBITS_PER_CYCLE;
    }

  ret.foot = t->pos.foot + ixy.y;

  return (ret);

}

static IXY
one_orbit_iof_to_ixy (Alttile t, IOF iof)
{
  IXY ret;

  ret.x = iof.orbit - t->pos.orbit;

  while (ret.x < 0)
    {
      ret.x += ORBITS_PER_CYCLE;
    }

  ret.y = iof.foot - t->pos.foot;

  return (ret);
}


static struct alttile_funcs one_orbit_funcs =
{one_orbit_ixy_to_iof,
 one_orbit_iof_to_ixy};

int is_on_alttile(Alttile t,IXY pos)
{
if (pos.x<0) return 0;
if (pos.y<0) return 0;
if (pos.x>=t->size.x) return 0;
if (pos.y>=t->size.y) return 0;
return 1;
}

void
info_alttile (View v, char *buf)
{
  Alttile t;
  char *type;

  IXY ixy;
  IOF s, e;

  if (!v->altview_private->current)
    return;

  t = v->altview_private->current;
  if (!t)
    return;

  switch (t->type)
    {
    case TILE_CONTIGUOUS:
      type = "Contiguous";
      break;
    case TILE_ONE_ORBIT:
      type = "One Orbit";
      break;
    default:
      type = "Unknown";
      break;
    }

  ixy.x = ixy.y = 0;
  s = t->funcs.ixy_to_iof (t, ixy);

  ixy = t->size;
  ixy.x--;
  ixy.y--;

  e = t->funcs.ixy_to_iof (t, ixy);


  sprintf (buf, "Cycle %d Feets %d-%d Orbits %d-%d Type %s",
           t->cycle,
           s.foot, e.foot,
           s.orbit, e.orbit,
           type);

}

void
alttile_destructor (Alttile a)
{
  if (a->data)
    free_ar_matrix (a->data);
  free (a);
}

void
alttile_ptr_destructor (Alttile a)
{
  if (a->data)
    free (a->data);
  free (a);
}

void
altview_private_destructor (View v)
{
  if (!v->altview_private)
    return;

  if (v->altview_private->tiles)
    Obj_Delete (v->altview_private->tiles);
  if (v->altview_private->current)
    Obj_Delete (v->altview_private->current);
  if (v->altview_private->buffered)
    Obj_Delete (v->altview_private->buffered);
  v->altview_private->tiles = NULL;
  v->altview_private->buffered = NULL;
  v->altview_private->current = NULL;
  free (v->altview_private);

}


void
altview_private_init (View v)
{
  v->altview_private = (Altview_private *) malloc (sizeof (Altview_private));
  v->altview_private->tiles = CREATE_LIST (Alttile);
  v->altview_private->current = NULL;
  v->altview_private->buffered = NULL;
}


Alttile
create_alttile_from_midr (Midr m, int cycle, int type)
{
  float latmax = -100.0;
  float latmin = 100.0;
  IOF iof;
  IXY ixy;

  int s = cycle_to_sorbit (cycle);
  int e = cycle_to_eorbit (cycle);

  int sf = NFEET;
  int ef = -NFEET;

  int i;
  Alttile t = NULL;

  IXY size;


  {
    int o[4];
    XY xy[4];
    LL ll;
    int err;

#define OF_UL 0
#define OF_UR 1
#define OF_LL 2
#define OF_LR 3

    for (i = 0; i < 4; ++i)
      {
        xy[i].x = xy[i].y = 0.0;
        err = -1;

        if (i & 1)
          {
            xy[i].x = (float) m->max_x;
            err = 1;
          }

        if (i & 2)
          xy[i].y = (float) m->max_y;

        ll = m->funcs.xy_to_ll (m, xy[i]);

        o[i] = find_nearest_orbit_to (ll, cycle, err);

        if (!o[i])
          return (t);


        if (ll.lat > latmax)
          latmax = ll.lat;
        if (ll.lat < latmin)
          latmin = ll.lat;
      }

/* Now fix up the wrapping */

    switch (type)
      {
      case TILE_CONTIGUOUS:
        /* Use next cycle to finish off */

        if (o[OF_UL] > o[OF_UR])
          {
            ll = m->funcs.xy_to_ll (m, xy[OF_UR]);
            o[OF_UR] = find_nearest_orbit_to (ll, cycle + 1, err);
            if (!o[OF_UR])
              return (t);
          }

        if (o[OF_LL] > o[OF_LR])
          {
            ll = m->funcs.xy_to_ll (m, xy[OF_LR]);
            o[OF_LR] = find_nearest_orbit_to (ll, cycle + 1, err);
            if (!o[OF_LR])
              return (t);
          }

        break;
      case TILE_ONE_ORBIT:     /* Bang it upstairs for the calculation */
        if (o[OF_UL] > o[OF_UR])
          o[OF_UR] += ORBITS_PER_CYCLE;
        if (o[OF_LL] > o[OF_LR])
          o[OF_LR] += ORBITS_PER_CYCLE;
        break;
      }


    for (i = 0; i < 4; ++i)
      {
        fprintf (MOAN, "build_tile: adjusted_orbits are %f %f %d\n",
                 xy[i].x, xy[i].y, o[i]);
      }

/* Figure the bounds */

    s = o[OF_UL];
    if (s > o[OF_LL])
      {
        s = o[OF_LL];
      }

    e = o[OF_UR];
    if (e < o[OF_LR])
      {
        e = o[OF_LR];
      }

    t = CREATE_OBJ (Alttile);


    t->size.x = (e - s) + 1;
    t->type = type;
    t->cycle = cycle;

    t->pos.orbit = s;

    switch (type)
      {
      case TILE_CONTIGUOUS:
        /* Use next cycle to finish off */

        t->funcs = contiguous_funcs;

        break;
      case TILE_ONE_ORBIT:     /* Bang it upstairs for the calculation */
        t->funcs = one_orbit_funcs;

        if (e > cycle_to_eorbit (cycle))
          e -= ORBITS_PER_CYCLE;
        break;
      }


  }


  fprintf (MOAN, "feet: Midr covers altimetry orbits %d to %d\n", s, e);
  fprintf (MOAN, "feet: Midr covers latitudes %f to %f\n", latmin, latmax);


/* Ok now find the valid feetprints */
  {
    alt_rec *feet = (alt_rec *) malloc (sizeof (alt_rec) * NFEET);
    alt_rec *ptr;
    FILE *alt_file;
    int f;

    for (i = 0; i < t->size.x; ++i)
      {
        {
          char buf[1024];

          ixy.y = 0;
          ixy.x = i;

          iof = t->funcs.ixy_to_iof (t, ixy);

          if ((i % 50) == 0)
            fprintf (MOAN,
                     "build_tile: scanning offset %d/%d -> orbit %d\n",
                     ixy.x, t->size.x, iof.orbit);


          if (!get_alt_filename (iof.orbit, buf))
            {
              Obj_Delete (t);
              t = NULL;
              return (t);
            }

          alt_file = fopen (buf, "r");
          if (!alt_file)
            {
              fprintf (MOAN, "alt: can't open %s for reading\n", buf);
            }

        }
        bzero (feet, sizeof (alt_rec) * NFEET);

        if (alt_file)
          {
            fread (feet, sizeof (alt_rec), NFEET, alt_file);
            fclose (alt_file);
          }


        {
          int n = NFEET;
          alt_rec *ptr = feet;
          while (n--)
            {

              BO_F_FTOM (&ptr->pos.lat, 2);
              BO_F_FTOM (&ptr->rad, 1);
              BO_I_FTOM (&ptr->flags, 1);
              ptr++;
            }
        }


        ptr = feet;
        for (f = 0; f < NFEET; ++f)
          {

            if (ptr->rad > 0.0)
              {
                if ((ptr->pos.lat <= latmax) && (ptr->pos.lat >= latmin))
                  {

                    if (f < sf)
                      sf = f;
                    if (f > ef)
                      ef = f;
                  }
              }

            ptr++;
          }

      }
    if (sf > ef)
      {
        fprintf (MOAN, "feet: No applicable atimetry found\n");
        Obj_Delete (t);
        t = NULL;
        return (t);
      }


    ef -= ALT_FIDDLE_FOOT;
    sf -= ALT_FIDDLE_FOOT;

    free (feet);
  }

  fprintf (MOAN, "feet: Midr covers feet %d to %d\n", sf, ef);




  t->size.y = (ef - sf) + 1;

  t->pos.foot = sf;

  t->data = create_ar_matrix (t->size.x, t->size.y);

/* Ok now copy in the valid feetprints */
  {
    alt_rec *feet = (alt_rec *) malloc (sizeof (alt_rec) * NFEET);
    alt_rec *ptr;
    alt_rec **dptr = t->data;
    int n = 0;

    FILE *alt_file;

    for (i = 0; i < t->size.x; ++i)
      {

        {
          char buf[1024];

          ixy.y = 0;
          ixy.x = i;

          iof = t->funcs.ixy_to_iof (t, ixy);

          if ((i % 50) == 0)
            fprintf (MOAN,
                     "build_tile: reading offset %d/%d -> orbit %d\n",
                     ixy.x, t->size.x, iof.orbit);


          if (!get_alt_filename (iof.orbit, buf))
            {
              Obj_Delete (t);
              t = NULL;

              return (t);
            }

          alt_file = fopen (buf, "r");
          if (!alt_file)
            {
              fprintf (MOAN, "alt: can't open %s for reading\n", buf);
            }

        }

        bzero (feet, sizeof (alt_rec) * NFEET);
        if (alt_file)
          {
            fread (feet, sizeof (alt_rec), NFEET, alt_file);
            fclose (alt_file);
          }


        ptr = feet + (sf + ALT_FIDDLE_FOOT);

        {
          int n = t->size.y;
          alt_rec *cptr = ptr;
          while (n--)
            {

              BO_F_FTOM (&cptr->pos.lat, 2);
              BO_F_FTOM (&cptr->rad, 1);
              BO_I_FTOM (&cptr->flags, 1);
              cptr++;
            }
        }


        bcopy (ptr, *dptr, t->size.y * sizeof (alt_rec));
        dptr++;
        n += t->size.y;

      }


    free (feet);
    fprintf (MOAN, "alt: created alt_tile of %d feetprints\n", n);
  }
  t->serial = alttileserial++;


  return (t);

}

float 
evaluate_distance (LL l1, LL l2)
{
  float d1, d2;
  d1 = l1.lat - l2.lat;
  d1 *= d1;
  d2 = l1.lon - l2.lon;

  LON_WRAP (d2);

  d2 *= d2;
  d1 += d2;


  return (d1);
}

void
alt_find_nearest_in_tile (Alttile t, LL ll, IXY * ret)
{
  float md = 1.0e15;
  int x, y;
  alt_rec *ptr;
  float d;

  for (x = 0; x < t->size.x; ++x)
    {
      ptr = t->data[x];
      for (y = 0; y < t->size.y; ++y)
        {
          d = evaluate_distance (ll, ptr->pos);
          if (d < md)
            {
              md = d;
              ret->x = x;
              ret->y = y;
            }

          ptr++;
        }

    }


  return;
}

void
alt_find_nearest_n_in_tile (Alttile t, LL ll, int n, IXY * ret)
{
  float *md, *lmd;
  IXY *lret;
  float d;
  int x, y;
  alt_rec *ptr;
  int i;


  md = (float *) malloc (sizeof (float) * n);
  for (i = 0; i < n; ++i)
    md[i] = 1.0e15;
  lmd = md + n;
  lmd--;

  lret = ret + n;
  lret--;

  for (x = 0; x < t->size.x; ++x)
    {
      ptr = t->data[x];
      for (y = 0; y < t->size.y; ++y)
        {
          d = evaluate_distance (ll, ptr->pos);

          if (d < *lmd)
            {
              i = n - 2;
              while ((i >= 0) && (d < md[i]))
                i--;
              i++;

              /* md[i] is now the element that needs replacing */
              /* How many need moving ? j=(n-1)-i */

              {
                int j = (n - 1) - i;
                float *mdp = lmd;
                IXY *ixp = lret;

                while (j--)
                  {
                    *(ixp) = *(ixp - 1);
                    ixp--;
                    *(mdp) = *(mdp - 1);
                    mdp--;
                  }

                /* And stick the value in the slot */

                *mdp = d;
                ixp->x = x;
                ixp->y = y;

              }



            }

          ptr++;
        }

    }

  free (md);
  return;
}

int
check_nearest_m_of_n_in_tile (Alttile t, LL ll, int m, int n, IXY * ret)
{
  alt_rec *ptr;
  int i;
  float cd, d;


  cd = 0.0;

/* Find furthest chosen */
  for (i = 0; i < m; ++i)
    {
      d = evaluate_distance (ll, t->data[ret[i].x][ret[i].y].pos);
      if (d > cd)
        cd = d;
    }


/* is it bigger than any of the others? */

  for (i = m; i < n; ++i)
    {

      d = evaluate_distance (ll, t->data[ret[i].x][ret[i].y].pos);
      if (d < cd)
        return 0;

    }

  return 1;



}


Valuetile
alt_make_value_tile (Alttile m)
{
  Valuetile r;
  int i, j;

  if (!m)
    return ((Valuetile) NULL);

  r = create_valuetile_with_borders (m->size);

  for (i = 0; i < m->size.x; ++i)
    {
      alt_rec *md = m->data[i];
      value_rec *rd = (r->data[i + 1]) + 1;

      for (j = 0; j < m->size.y; ++j)
        {
          rd->value = md->rad;
          rd->pos = md->pos;

          rd++;
          md++;

        }

    }

  return (r);
}



void
alt_find_range (Alttile t, float *min, float *max)
{
  int x, y;

  *min = 1.0e50;
  *max = -1.0e50;

  for (x = 0; x < t->size.x; ++x)
    {
      alt_rec *r = t->data[x];

      for (y = 0; y < t->size.y; ++y)
        {

          if (r->rad != 0.0)
            {
              if (r->rad < *min)
                *min = r->rad;
              if (r->rad > *max)
                *max = r->rad;
            }

          r++;
        }


    }

}
