/*
 * $Log: contor.c,v $
 * Revision 14.0  1998/07/31 19:46:47  root
 * \#makecheckin
 *
 * Revision 13.0  1998/07/31 19:46:30  root
 * \#makecheckin
 *
 * Revision 12.0  1998/07/31 09:58:26  root
 * \#makecheckin
 *
 * Revision 11.0  1998/05/02 23:03:44  root
 * \#makecheckin
 *
 * Revision 10.1  1997/10/22 18:35:49  root
 * \#makecheckin
 *
 * Revision 10.0  1997/10/22 18:33:55  root
 * \#makecheckin
 *
 * Revision 6.0  1997/07/04  00:32:42  root
 * \#makecheckin
 *
 * Revision 5.1  1997/07/04 00:31:03  root
 * \#makecheckin
 *
 * Revision 5.0  1997/07/03 15:05:48  root
 * \#makecheckin
 *
 * Revision 4.0  1997/07/01 11:58:59  root
 * \#makecheckin
 *
 * Revision 3.0  1997/06/23 12:51:50  root
 * \#makecheckin
 *
 * Revision 2.0  1997/06/23 09:04:44  root
 * \#makecheckin
 *
 * Revision 1.9  1997/06/18 15:15:32  root
 * \#makecheckin
 *
 * Revision 1.8  1997/06/15 23:47:49  root
 * \#makecheckin
 *
 */

static char rcsid[] = "$Id: contor.c,v 14.0 1998/07/31 19:46:47 root Exp root $";

#include "mgn.h"


/* This contouring code lifted from my jwg library */
/* begin lifted */


typedef struct contour_struct
  {
    value_rec **dcol;

    unsigned char *flags;
    unsigned char **fcol;

    int x, y;
    int x1, y1;

    IXY *current_line_data;
    int current_line_npts;
    int current_line_size;

    View v;
  }
 *Contour;


#define INRANGE(l1,l2,c) ( ((c)<(l2)) && ((c)>=(l1)) )
#define RANGEX(c,x)	( ((x)>0) ? ( ((x)<((c)->x1)) ? (x):((c)->x1)) : 1)
#define RANGEY(c,y)	(((y)>0) ?  ( ((y)<((c)->y1)) ? (y):((c)->y1)) : 1)

/* Yes I know these are wrong, but the pointer code in find_possible */
/* is best left alone [I know, I wrote it], so we'll fix it with a */
/* transform in add_point */

#define FLAGS(c,p) ((c->fcol[p.y])[p.x])
#define DATA(c,p) ((c->dcol[p.y])[p.x].value)
#define POS(c,p) ((c->dcol[p.y])[p.x].pos)

#define DEF_LINE_SIZE 8192



void
info_contor_level (View v, char *buf)
{

  float *level = (float *) xv_get (v->frame, XV_KEY_DATA, CONTOUR_LEVEL_KEY);
  if (!level)
    return;

  sprintf (buf, "%.2f km", *level);

}

void
info_contor_step (View v, char *buf)
{
  float *step = (float *) xv_get (v->frame, XV_KEY_DATA, CONTOUR_STEP_KEY);
  if (!step)
    return;

  if (*step == 0.0)
    {
      strcpy (buf, "None(off)");
    }
  else
    {
      sprintf (buf, "%.2f km", *step);
    }

}

static void
cont_add_point (Contour c, LL pos)
{
  IXY *ptr;
  int n;

  if (c->current_line_npts == c->current_line_size)
    {
      /* uh-oh we're about to over flow */

      ptr = c->current_line_data;


      n = c->current_line_size;
      fprintf (MOAN, "contour: Increasing maximum line size from %d to %d\n",
               n, n << 1);

      c->current_line_size = n * 2;

      c->current_line_data = malloc (sizeof (IXY) * n * 2);
      bcopy (ptr, c->current_line_data, sizeof (IXY) * n);

      free (ptr);

    }


  c->current_line_data[c->current_line_npts] =
    ll_to_ixy (c->v, pos);




  c->current_line_npts++;

}

static unsigned char
find_direction (IXY p1, IXY p2)
{
  p2.x -= p1.x;
  p2.y -= p1.y;

  switch (p2.y)
    {
    case -1:
      switch (p2.x)
        {
        case -1:
          return (1);
        case 0:
          return (2);
        case 1:
          return (4);
        }
    case 0:
      switch (p2.x)
        {
        case -1:
          return (128);
        case 1:
          return (8);
        }
    case 1:
      switch (p2.x)
        {
        case -1:
          return (64);
        case 0:
          return (32);
        case 1:
          return (16);
        }
    }


  fprintf (MOAN, "contour: Contouring cockup type 3 (very fatal) ...\n");
  fprintf (MOAN, "contour: bombing ...\n");
  abort ();

}


static int
points_marked (Contour c, IXY p1, IXY p2)
{
  return (c->fcol[p1.y])[p1.x] & find_direction (p1, p2);
}

static void
mark_points (Contour c, IXY p1, IXY p2)
{
  (c->fcol[p1.y])[p1.x] |= find_direction (p1, p2);
  (c->fcol[p2.y])[p2.x] |= find_direction (p2, p1);
}



static
trace (Contour c, IXY p1, IXY p2, IXY pl, float level)
{
  IXY pn;
  float v1, v2, vn;
  IXY ipos;
  LL pos;
  float r;


  c->current_line_npts = 0;

  v1 = DATA (c, p1);
  v2 = DATA (c, p2);

  while (1)
    {

      /*
       * printf("(%d,%d,%f) (%d,%d,%f) (%d,%d,%f)\n", p1.x, p1.y, DATA(c, p1),
       * p2.x, p2.y, DATA(c, p2), pl.x, pl.y, DATA(c, pl));
       */

      r = (v2 - v1);
      if (r == 0.0)
        {
          r = 0.5;
        }
      else
        {
          r = (level - v1) / r;
        }

      {
        IXY l1, l2;
        LL pos2, pos1;
        int u = 0;

        l1.x = RANGEX (c, p2.x);
        l1.y = RANGEY (c, p2.y);

        if (DATA (c, l1) == 0.0)
          {
            u++;
          }
        else
          {
            pos = pos2 = POS (c, l1);
          }

        l2.x = RANGEX (c, p1.x);
        l2.y = RANGEY (c, p1.y);

        if (DATA (c, l2) == 0.0)
          {
            u++;
          }
        else
          {
            pos = pos1 = POS (c, l2);
          }

        if (!u)
          {
            pos.lat = r * pos2.lat;
            pos.lon = r * pos2.lon;

            r = 1.0 - r;
            pos.lat += r * pos1.lat;
            pos.lon += r * pos1.lon;
          }
        if (u < 2)
          cont_add_point (c, pos);

      }




      if (DATA (c, p1) > level)
        {
          printf ("Contouring cockup type 1 ((%d,%d),%f)-%f-((%d,%d),%f)\n",
                  p1.x, p1.y, DATA (c, p1), level, p2.x, p2.y, DATA (c, p2));
          view_draw_line (c->v, c->current_line_data, c->current_line_npts);


          return;
        }
      if (DATA (c, p2) <= level)
        {
          printf ("Contouring cockup type 2 ((%d,%d),%f)-%f-((%d,%d),%f)\n",
                  p1.x, p1.y, DATA (c, p1), level, p2.x, p2.y, DATA (c, p2));
          view_draw_line (c->v, c->current_line_data, c->current_line_npts);


          return;
        }

      if (points_marked (c, p1, p2))
        {

          view_draw_line (c->v, c->current_line_data, c->current_line_npts);


          return;
        }

      mark_points (c, p1, p2);

      pn.x = p1.x + p2.x - pl.x;
      pn.y = p1.y + p2.y - pl.y;

      vn = DATA (c, pn);

      if (vn <= level)
        {
          pl = p1;
          p1 = pn;
          v1 = vn;
        }
      else
        {
          pl = p2;
          p2 = pn;
          v2 = vn;
        }


    }

}





static
possible (Contour c, int x1, int y1, int x2, int y2, int xl, int yl, float level)
{
  IXY p1, p2, pl;

  p1.x = x1;
  p1.y = y1;
  p2.x = x2;
  p2.y = y2;
  pl.x = xl;
  pl.y = yl;

  if (points_marked (c, p1, p2))
    return;

  trace (c, p1, p2, pl, level);


}


static
contour (Contour c, float level)
{
  int x, y;
  value_rec **c0, **c1;
  value_rec *r00, *r10, *r11, *r01;


  c1 = c0 = c->dcol;
  c1--;

  for (y = 0; y < c->y; ++y)
    {
      r00 = r01 = *(c0);
      r01--;
      if (y)
        {
          r10 = r11 = *(c1);
          r11--;
        }
      for (x = 0; x < c->x; ++x)
        {

          if (x)
            {
              if (INRANGE (r00->value, r01->value, level))
                possible (c, x, y, x - 1, y, x, y - 1, level);
              if (INRANGE (r01->value, r00->value, level))
                possible (c, x - 1, y, x, y, x - 1, y + 1, level);
            }
          if (y)
            {
              if (INRANGE (r00->value, r10->value, level))
                possible (c, x, y - 1, x, y, x - 1, y, level);
              if (INRANGE (r10->value, r00->value, level))
                possible (c, x, y, x, y - 1, x + 1, y - 1, level);
            }
          r00++;
          r10++;
          r11++;
          r01++;
        }
      c0++;
      c1++;
    }

}



void
do_contour (View v, Valuetile t, float level, int side)
{
  Contour c = (Contour) malloc (sizeof (struct contour_struct));
  int i;
  float hug = 1.0e10 * (float) side;



  /* Invert these as minor axis is (real y) t->ny */
  c->x1 = c->x = t->size.y;
  c->y1 = c->y = t->size.x;

  /* Now the code contructs borders and the like */


  c->dcol = t->data;


  c->flags = (unsigned char *) malloc (c->x * c->y);
  bzero (c->flags, c->x * c->y);

  c->fcol = (unsigned char **) malloc (c->y * sizeof (unsigned char *));

  c->fcol[0] = c->flags;
  for (i = 1; i < c->y; ++i)
    c->fcol[i] = c->fcol[i - 1] + c->x;

  c->x1--;
  c->y1--;

  for (i = 0; i < c->y; ++i)
    {
      (c->dcol[i])[0].value = hug;
      (c->dcol[i])[c->x1].value = hug;

    }
  for (i = 0; i < c->x; ++i)
    {
      (c->dcol[0])[i].value = hug;
      (c->dcol[c->y1])[i].value = hug;
    }

  c->x1--;
  c->y1--;

  c->current_line_size = DEF_LINE_SIZE;
  c->current_line_data = (IXY *) malloc (sizeof (IXY) * DEF_LINE_SIZE);

  c->v = v;

  contour (c, level);


  free (c->flags);
  free (c->fcol);
  free (c->current_line_data);
  free (c);


}



/* end lifted */

value_rec **
create_value_rec_matrix (int x, int y)
{
  value_rec *data = (value_rec *) malloc (sizeof (alt_rec) * x * y);
  value_rec **ptrs = (value_rec **) malloc (sizeof (alt_rec *) * x);

  value_rec *p1 = data;
  value_rec **p2 = ptrs;

  while (x--)
    {
      *p2 = p1;
      p1 += y;
      p2++;
    }

  return (ptrs);
}


void
free_value_rec_matrix (value_rec ** base)
{
  free (*base);
  free (base);
}


Valuetile
create_valuetile_with_borders (IXY size)
{
  Valuetile r;

  size.x += 2;
  size.y += 2;

  r = CREATE_OBJ (Valuetile);

  r->size = size;
  r->data = create_value_rec_matrix (size.x, size.y);

  return (r);
}

void
valuetile_destructor (Valuetile t)
{
  free_value_rec_matrix (t->data);
  free (t);
}


void
valuetile_find_range (Valuetile t, float *min, float *max)
{
  int x, y;

  *min = 1.0e50;
  *max = -1.0e50;

  for (x = 1; x < (t->size.x - 1); ++x)
    {
      value_rec *r = t->data[x];
      r++;

      for (y = 2; y < t->size.y; ++y)
        {

          if (r->value < *min)
            *min = r->value;
          if (r->value > *max)
            *max = r->value;

          r++;
        }


    }

}
