/*
 * $Log: graphics.c,v $
 * Revision 14.0  1998/07/31 19:46:47  root
 * \#makecheckin
 *
 * Revision 13.0  1998/07/31 19:46:30  root
 * \#makecheckin
 *
 * Revision 12.0  1998/07/31 09:58:26  root
 * \#makecheckin
 *
 * Revision 11.0  1998/05/02 23:03:44  root
 * \#makecheckin
 *
 * Revision 10.1  1997/10/22 18:35:48  root
 * \#makecheckin
 *
 * Revision 10.0  1997/10/22 18:33:55  root
 * \#makecheckin
 *
 * Revision 6.0  1997/07/04  00:32:42  root
 * \#makecheckin
 *
 * Revision 5.0  1997/07/03 15:05:48  root
 * \#makecheckin
 *
 * Revision 4.0  1997/07/01 11:58:59  root
 * \#makecheckin
 *
 * Revision 3.1  1997/07/01 11:58:46  root
 * \#makecheckin
 *
 * Revision 3.0  1997/06/23 12:51:50  root
 * \#makecheckin
 *
 * Revision 2.0  1997/06/23 09:04:44  root
 * \#makecheckin
 *
 * Revision 1.7  1997/06/23 08:53:03  root
 * \#makecheckin
 *
 * Revision 1.6  1997/06/18 15:15:32  root
 * \#makecheckin
 *
 * Revision 1.5  1997/06/15 23:47:48  root
 * \#makecheckin
 *
 */

static char rcsid[] = "$Id: graphics.c,v 14.0 1998/07/31 19:46:47 root Exp root $";

#include "mgn.h"

BEGIN_OBJECT (MYFont)
     Display *display;
     Font font;
     XFontStruct *fs;
     char *name;
END_OBJECT (MYFont)

     static void myfont_destructor (MYFont m)
{
  if (m->name)
    free (m->name);
  XFreeFont (m->display, m->fs);
  free (m);
}

CREATOR (MYFont) = Obj_DefaultCreator;
DESTRUCTOR (MYFont) = myfont_destructor;

MYFont_list myfonts = NULL;


static MYFont
get_myfont (Display * display, char *name)
{
  MYFont ret;

  if (!myfonts)
    {
      myfonts = CREATE_LIST (MYFont);
    }

  ret = myfonts->head;
  while (ret)
    {

      if (strcmp (ret->name, name) == 0)
        {
          return (ret);
        }

      ret = ret->next;
    }

  ret = CREATE_OBJ (MYFont);

  ret->name = strdup (name);
  ret->font = XLoadFont (display, name);
  ret->fs = XQueryFont (display, ret->font);
  ret->display = display;

  Obj_Insert (myfonts, ret, ObjInsertTail);

  return (ret);
}


void
view_set_color (View v, int c)
{
  XSetForeground (v->display, v->gc, c);
}



void
view_draw_line (View v, IXY * line, int n)
{
  XPoint *points = malloc (sizeof (XPoint) * n);

  {
    IXY *a = line;
    XPoint *b = points;
    int m = n;

    while (m--)
      {
        b->x = a->x;
        b->y = a->y;
        a++;
        b++;
      }

  }

  XDrawLines (v->display, v->window, v->gc, points, n, CoordModeOrigin);

  free (points);
}

void
view_DrawLine (View v, int x1, int y1, int x2, int y2)
{
  XDrawLine (v->display, v->window, v->gc, x1, y1, x2, y2);


}

void
view_fill_centered_rectangle (View v, IXY pos, IXY size)
{

  XFillRectangle (v->display, v->window, v->gc,
                  pos.x - size.x, pos.y - size.y, 2 * size.x, 2 * size.y);

}
void
view_draw_box (View v, IXY pos, int n)
{

  XDrawRectangle (v->display, v->window, v->gc,
                  pos.x - n, pos.y - n, 2 * n, 2 * n);

}

void
view_draw_boxes (View v, IXY * pos, int w, int n)
{
  XRectangle *r = (XRectangle *) malloc (sizeof (XRectangle) * n);
  XRectangle *p = r;
  int m = n;

  while (m--)
    {
      p->x = pos->x - w;
      p->y = pos->y - w;
      p->width = 2 * w;
      p->height = 2 * w;

      p++;
      pos++;
    }


  XDrawRectangles (v->display, v->window, v->gc, r, n);

  free (r);

}

void
view_draw_points (View v, IXY * line, int n)
{
  XPoint *points = malloc (sizeof (XPoint) * n);

  {
    IXY *a = line;
    XPoint *b = points;
    int m = n;

    while (m--)
      {
        b->x = a->x;
        b->y = a->y;
        a++;
        b++;
      }

  }

  XDrawPoints (v->display, v->window, v->gc, points, n, CoordModeOrigin);

  free (points);
}

void
view_draw_point (View v, IXY pos)
{
  XDrawPoint (v->display, v->window, v->gc, pos.x, pos.y);
}

void
view_draw_string (View v, IXY pos, char *s)
{
  XDrawString (v->display, v->window, v->gc, pos.x, pos.y, s, strlen (s));
}

void
view_set_font (View v, int height)
{
  char fontname[1024];
  MYFont f;

  sprintf (fontname, "lucidasanstypewriter-%d", height);
  f = get_myfont (v->display, fontname);

  XSetFont (v->display, v->gc, f->font);

}

int
view_get_textwidth (View v, int h, char *s)
{
  char fontname[1024];
  MYFont f;
  int width;

  sprintf (fontname, "lucidasanstypewriter-%d", h);
  f = get_myfont (v->display, fontname);


  width = XTextWidth (f->fs, s, strlen (s));

  return (width);
}

int
view_onscreen (View v, IXY pos)
{
  if (pos.x < 0)
    return (0);
  if (pos.x >= v->wx)
    return (0);
  if (pos.y < 0)
    return (0);
  if (pos.y >= v->wy)
    return (0);

  return (1);
}
