/*
 * $Log: grid.c,v $
 * Revision 14.0  1998/07/31 19:46:47  root
 * \#makecheckin
 *
 * Revision 13.0  1998/07/31 19:46:30  root
 * \#makecheckin
 *
 * Revision 12.0  1998/07/31 09:58:26  root
 * \#makecheckin
 *
 * Revision 11.0  1998/05/02 23:03:44  root
 * \#makecheckin
 *
 * Revision 10.0  1997/10/22 18:33:55  root
 * \#makecheckin
 *
 * Revision 6.0  1997/07/04  00:32:42  root
 * \#makecheckin
 *
 * Revision 5.0  1997/07/03 15:05:48  root
 * \#makecheckin
 *
 * Revision 4.0  1997/07/01 11:58:59  root
 * \#makecheckin
 *
 * Revision 3.0  1997/06/23 12:51:50  root
 * \#makecheckin
 *
 * Revision 2.0  1997/06/23 09:04:44  root
 * \#makecheckin
 *
 * Revision 1.11  1997/06/23 08:53:03  root
 * \#makecheckin
 *
 * Revision 1.10  1997/06/18 15:15:32  root
 * \#makecheckin
 *
 * Revision 1.9  1997/06/15 23:47:48  root
 * \#makecheckin
 *
 */

static char rcsid[] = "$Id: grid.c,v 14.0 1998/07/31 19:46:47 root Exp root $";

#include "mgn.h"

#define GRID_STEPS 40
#define GRID_TEXT_HEIGHT 12

void
info_grid_step (View v, char *buf)
{
  float *grid = (float *) xv_get (v->frame, XV_KEY_DATA, GRID_KEY);
  if (!grid)
    return;

  if (*grid == 0.0)
    {
      strcpy (buf, "None(off)");
    }
  else
    {
      sprintf (buf, "%.2f deg", *grid);
    }

}



void
grid_repaint (View v)
{
  float *grid = (float *) xv_get (v->frame, XV_KEY_DATA, GRID_KEY);
  float g = *grid;
  IXY *line;

  IXY ixy;
  LL ll, max, min;

  int i;

  switch (v->mode)
    {
    case V_NORMAL:
      break;
    case V_BROWSING:
      g *= 10.0;
      break;
    default:
      return;
    }


  max.lon = max.lat = -720.0;
  min.lon = min.lat = 720.0;

  if (g == 0.0)
    return;

  line = malloc (sizeof (ixy) * (GRID_STEPS + 10));

  for (i = 0; i < 4; ++i)
    {

      ixy.x = ixy.y = 0;
      if (i & 1)
        ixy.x += v->wx;
      if (i & 2)
        ixy.y += v->wy;


      ll = ixy_to_ll (v, ixy);

      if (ll.lat > max.lat)
        max.lat = ll.lat;
      if (ll.lat < min.lat)
        min.lat = ll.lat;
      if (ll.lon > max.lon)
        max.lon = ll.lon;
      if (ll.lon < min.lon)
        min.lon = ll.lon;

    }

  max.lat = g * (ceil (max.lat / g) + 1.1);
  max.lon = g * (ceil (max.lon / g) + 1.1);
  min.lat = g * (floor (min.lat / g) - 1.0);
  min.lon = g * (floor (min.lon / g) - 1.0);


  view_set_color (v, MGN_WHITE);
  view_set_font (v, GRID_TEXT_HEIGHT);

/* Draw lines of latitude first */
  for (ll.lat = min.lat; ll.lat < max.lat; ll.lat += g)
    {
      int n = 0;
      int i;



      float lstep = (max.lon - min.lon) / GRID_STEPS;

      for (ll.lon = min.lon + g; ll.lon < max.lon; ll.lon += lstep)
        {
          line[n++] = ll_to_ixy (v, ll);
        }
      for (i = 0; i < n; ++i)
        {
          if (view_onscreen (v, line[i]))
            {
              char buf[100];
              IXY pos = line[i];
              IXY size;

              sprintf (buf, "%.1f", ll.lat);

              size.x = view_get_textwidth (v, GRID_TEXT_HEIGHT, buf);
              size.y = GRID_TEXT_HEIGHT;

              pos.x -= size.x >> 1;
              pos.y -= size.y >> 1;

              if (view_onscreen (v, pos))
                {
/*
   view_set_color (v, MGN_BLACK);
   view_fill_centered_rectangle(v,pos,size);
   view_set_color (v, MGN_WHITE);
 */
                  view_draw_string (v, pos, buf);
                  i = n;
                }
            }
        }


      view_draw_line (v, line, n);

    }

  for (ll.lon = min.lon + g; ll.lon < max.lon; ll.lon += g)
    {
      int n = 0;
      int i;


      float lstep = (max.lat - min.lat) / GRID_STEPS;

      for (ll.lat = max.lat; ll.lat > min.lat; ll.lat -= lstep)
        {
          line[n++] = ll_to_ixy (v, ll);
        }

      for (i = 0; i < n; ++i)
        {
          if (view_onscreen (v, line[i]))
            {
              char buf[100];
              IXY pos = line[i];
              IXY size;

              sprintf (buf, "%.1f", ll.lon);

              size.x = view_get_textwidth (v, GRID_TEXT_HEIGHT, buf);
              size.y = GRID_TEXT_HEIGHT;

              pos.x -= size.x >> 1;
              pos.y -= size.y >> 1;

              if (view_onscreen (v, pos))
                {
/*
   view_set_color (v, MGN_BLACK);
   view_fill_centered_rectangle(v,pos,size);
   view_set_color (v, MGN_WHITE);
 */

                  view_draw_string (v, pos, buf);
                  i = n;
                }
            }
        }



      view_draw_line (v, line, n);

    }



  free (line);
}




static void
grid_notify (Menu m, Menu_item mi)
{
  float *val = (float *) xv_get (mi, XV_KEY_DATA, GRID_KEY);
  View v = (View) xv_get (mi, XV_KEY_DATA, VIEW_KEY);
  float *grid = (float *) xv_get (v->frame, XV_KEY_DATA, GRID_KEY);

  xv_set (m, MENU_DEFAULT_ITEM, mi, 0);

  *grid = *val;

  ext_view_repaint (v);

}

void
grid_base_menu (View v)
{
  Menu m;
  char buf[80];
  int d, b, i;
  float *grid = malloc (sizeof (float));
  Menu_item mi;
  int otf[] =
  {1, 2, 5};

  *grid = 0.0;

  m = xv_create (0, MENU,
                 MGN_PIN (v, "Grid")
                 0);

  {
    float *val = malloc (sizeof (float));
    *val = 0.0;

    mi = xv_create (0, MENUITEM,
                    MENU_STRING, "Off",
                    MENU_NOTIFY_PROC, grid_notify,
                    XV_KEY_DATA, GRID_KEY, val,
                    XV_KEY_DATA, VIEW_KEY, v,
                    0);

    xv_set (m, MENU_APPEND_ITEM, mi, 0);
    xv_set (m, MENU_DEFAULT_ITEM, mi, 0);
  }



  b = 1;
  for (d = 0; d < 4; ++d)
    {
      for (i = 0; i < 3; ++i)
        {
          float *val = malloc (sizeof (float));

          *val = (float) b;
          *val *= (float) otf[i];
          *val *= 0.01;

          sprintf (buf, "%.2f deg", *val);

          mi = xv_create (0, MENUITEM,
                          MENU_STRING, strdup (buf),
                          MENU_RELEASE,
                          MENU_NOTIFY_PROC, grid_notify,
                          XV_KEY_DATA, GRID_KEY, val,
                          XV_KEY_DATA, VIEW_KEY, v,
                          0);

          xv_set (m, MENU_APPEND_ITEM, mi, 0);



        }

      b *= 10;
    }

  xv_set (v->menu, MENU_PULLRIGHT_ITEM, "Grid", m, 0);
  xv_set (v->frame, XV_KEY_DATA, GRID_KEY, grid, 0);
}
