/*
 * $Log: image.c,v $
 * Revision 14.0  1998/07/31 19:46:47  root
 * \#makecheckin
 *
 * Revision 13.0  1998/07/31 19:46:30  root
 * \#makecheckin
 *
 * Revision 12.0  1998/07/31 09:58:26  root
 * \#makecheckin
 *
 * Revision 11.0  1998/05/02 23:03:44  root
 * \#makecheckin
 *
 * Revision 10.1  1997/10/22 18:35:48  root
 * \#makecheckin
 *
 * Revision 10.0  1997/10/22 18:33:55  root
 * \#makecheckin
 *
 * Revision 6.0  1997/07/04  00:32:42  root
 * \#makecheckin
 *
 * Revision 5.0  1997/07/03 15:05:48  root
 * \#makecheckin
 *
 * Revision 4.0  1997/07/01 11:58:59  root
 * \#makecheckin
 *
 * Revision 3.0  1997/06/23 12:51:50  root
 * \#makecheckin
 *
 * Revision 2.0  1997/06/23 09:04:44  root
 * \#makecheckin
 *
 * Revision 1.10  1997/06/18 22:41:11  root
 * \#makecheckin
 *
 * Revision 1.9  1997/06/18 22:11:09  root
 * \#makecheckin
 *
 * Revision 1.8  1997/06/18 21:30:32  root
 * \#makecheckin
 *
 * Revision 1.7  1997/06/18  15:15:32  root
 * \#makecheckin
 *
 * Revision 1.6  1997/06/15 23:47:47  root
 * \#makecheckin
 *
 */

static char rcsid[] = "$Id: image.c,v 14.0 1998/07/31 19:46:47 root Exp root $";

#include "mgn.h"

void
image_destructor (Image image)
{
  switch (image->type)
    {
    case MGN_NO_IMAGE:
      free (image);
      break;
    case MGN_XIMAGE:

      XDestroyImage (image->p.x.ximage);
      free (image);
      break;

    case MGN_MEMIMAGE:
      fprintf (MOAN, "image: can't free memimage\n");
      break;
    case MGN_SHXIMAGE:
      XShmDetach (image->p.shx.display, &image->p.shx.shm);
      XFreePixmap (image->p.shx.display, image->p.shx.pixmap);
      XDestroyImage (image->p.shx.ximage);
      shmdt (image->p.shx.shm.shmaddr);
      shmctl (image->p.shx.shm.shmid, IPC_RMID, 0);
      free (image);
      break;
    case MGN_NPSHXIMAGE:
      XShmDetach (image->p.npshx.display, &image->p.npshx.shm);
      XDestroyImage (image->p.npshx.ximage);
      shmdt (image->p.npshx.shm.shmaddr);
      shmctl (image->p.npshx.shm.shmid, IPC_RMID, 0);
      free (image);
      break;
    default:
      fprintf (MOAN, "image: can't free unknown image type\n");
      break;

    }


}



static void
shx_copyarea_to_x (struct Image_struct *i,
                   Display * d, Window win, GC gc,
                   int dx, int dy, int w, int h, int sx, int sy)
{

  XCopyArea (d, i->p.shx.pixmap, win, gc, dx, dy, w, h, sx, sy);

}



static struct image_funcs shx_funcs =
{shx_copyarea_to_x};


static void
no_image_copyarea_to_x (struct Image_struct *i,
                        Display * d, Window win, GC gc,
                        int dx, int dy, int w, int h, int sx, int sy)
{


}

static struct image_funcs no_image_funcs =
{no_image_copyarea_to_x};


static void
x_copyarea_to_x (struct Image_struct *i,
                 Display * d, Window win, GC gc,
                 int dx, int dy, int w, int h, int sx, int sy)
{
  XPutImage (d, win, gc, i->p.x.ximage, dx, dy, sx, sy, w, h);
}

static struct image_funcs x_funcs =
{x_copyarea_to_x};


static void
npshx_copyarea_to_x (struct Image_struct *i,
                     Display * d, Window win, GC gc,
                     int dx, int dy, int w, int h, int sx, int sy)
{
  XShmPutImage (d, win, gc, i->p.npshx.ximage, dx, dy, sx, sy, w, h, False);
}

static struct image_funcs npshx_funcs =
{npshx_copyarea_to_x};



Image
create_image (Create_image_args args)
{
  Image ret = CREATE_OBJ (Image);


  ret->type = args.type;

  switch (args.type)
    {
    case MGN_NO_IMAGE:
      ret->x = 0;
      ret->y = 0;
      ret->data = NULL;
      ret->bytesperline = 0;
      ret->funcs = no_image_funcs;
      return (ret);

    case MGN_XIMAGE:
      ret->data = (uchar *) malloc (args.x * args.y);

      ret->p.x.ximage = XCreateImage (args.p.x.display,
                                      DefaultVisual (args.p.shx.display,
                                        DefaultScreen (args.p.shx.display)),
                                      8, ZPixmap, 0, (char *) ret->data,
                                      args.x, args.y, 8, args.x);


      if (!ret->p.x.ximage)
        {
          free (ret->data);
          free (ret);
          return (NULL);
        }

      ret->bytesperline = ret->p.x.ximage->bytes_per_line;
      ret->x = args.x;
      ret->y = args.y;

      ret->funcs = x_funcs;


      return (ret);
    case MGN_MEMIMAGE:
      fprintf (MOAN, "image: Memimage type not yet supported\n");
      free (ret);
      return (NULL);
    case MGN_SHXIMAGE:
      ret->funcs = shx_funcs;
      ret->p.shx.display = args.p.shx.display;

      ret->p.shx.ximage = XShmCreateImage (args.p.shx.display,
                                           DefaultVisual (args.p.shx.display,
                                        DefaultScreen (args.p.shx.display)),
                         8, ZPixmap, NULL, &ret->p.shx.shm, args.x, args.y);

      ret->p.shx.shm.shmid = shmget (IPC_PRIVATE,
                                     ret->p.shx.ximage->bytes_per_line *
                                     ret->p.shx.ximage->height,
                                     IPC_CREAT | 0777);

      if (ret->p.shx.shm.shmid == -1)
        {
          fprintf (MOAN, "image: Failed to create shm segment of size %d\n",
             ret->p.shx.ximage->bytes_per_line * ret->p.shx.ximage->height);

          XDestroyImage (ret->p.shx.ximage);
          free (ret);
          return (NULL);
        }

      ret->p.shx.shm.shmaddr = ret->p.shx.ximage->data = shmat (ret->p.shx.shm.shmid, 0, 0);

      ret->data = (uchar *) ret->p.shx.ximage->data;
      ret->bytesperline = ret->p.shx.ximage->bytes_per_line;

      if ((long int) ret->p.shx.shm.shmaddr == -1)
        {
          fprintf (MOAN, "image: Could not attach shm segment\n");
          XDestroyImage (ret->p.shx.ximage);
          shmctl (ret->p.shx.shm.shmid, IPC_RMID, 0);
          free (ret);
          return (NULL);
        }


      ret->p.shx.shm.readOnly = False;

      if (!XShmAttach (args.p.shx.display, &ret->p.shx.shm))
        {
          fprintf (MOAN, "image: XShmAttach failed\n");
          XDestroyImage (ret->p.shx.ximage);
          shmdt (ret->p.shx.shm.shmaddr);
          shmctl (ret->p.shx.shm.shmid, IPC_RMID, 0);
          free (ret);
          return (NULL);
        }

      ret->p.shx.pixmap = XShmCreatePixmap (
                                      args.p.shx.display, args.p.shx.window,
                                    ret->p.shx.shm.shmaddr, &ret->p.shx.shm,
                                             args.x, args.y, 8);
      ret->x = args.x;
      ret->y = args.y;

      XSync (args.p.shx.display, False);

      shmctl (ret->p.shx.shm.shmid, IPC_RMID, 0);
      return (ret);
    case MGN_NPSHXIMAGE:
      ret->funcs = npshx_funcs;
      ret->p.npshx.display = args.p.npshx.display;

      ret->p.npshx.ximage = XShmCreateImage (args.p.npshx.display,
                                        DefaultVisual (args.p.npshx.display,
                                      DefaultScreen (args.p.npshx.display)),
                       8, ZPixmap, NULL, &ret->p.npshx.shm, args.x, args.y);

      ret->p.npshx.shm.shmid = shmget (IPC_PRIVATE,
                                       ret->p.npshx.ximage->bytes_per_line *
                                       ret->p.npshx.ximage->height,
                                       IPC_CREAT | 0777);

      if (ret->p.npshx.shm.shmid == -1)
        {
          fprintf (MOAN, "image: Failed to create shm segment of size %d\n",
          ret->p.npshx.ximage->bytes_per_line * ret->p.npshx.ximage->height);

          XDestroyImage (ret->p.npshx.ximage);
          free (ret);
          return (NULL);
        }

      ret->p.npshx.shm.shmaddr = ret->p.npshx.ximage->data = shmat (ret->p.npshx.shm.shmid, 0, 0);

      ret->data = (uchar *) ret->p.npshx.ximage->data;
      ret->bytesperline = ret->p.npshx.ximage->bytes_per_line;

      if ((long int) ret->p.npshx.shm.shmaddr == -1)
        {
          fprintf (MOAN, "image: Could not attach shm segment\n");
          XDestroyImage (ret->p.npshx.ximage);
          shmctl (ret->p.npshx.shm.shmid, IPC_RMID, 0);
          free (ret);
          return (NULL);
        }


      ret->p.npshx.shm.readOnly = False;

      if (!XShmAttach (args.p.npshx.display, &ret->p.npshx.shm))
        {
          fprintf (MOAN, "image: XShmAttach failed\n");
          XDestroyImage (ret->p.npshx.ximage);
          shmdt (ret->p.npshx.shm.shmaddr);
          shmctl (ret->p.npshx.shm.shmid, IPC_RMID, 0);
          free (ret);
          return (NULL);
        }

      ret->x = args.x;
      ret->y = args.y;

      XSync (args.p.shx.display, False);

      shmctl (ret->p.shx.shm.shmid, IPC_RMID, 0);
      return (ret);
    }

  fprintf (MOAN, "image: unknown image type requested in create_image\n");
  free (ret);
  return (NULL);


}

void
shrink_image (Image i, int s)
{
  int dy, sy;
  uchar *srptr, *drptr;

  srptr = drptr = i->data;
  dy = sy = 0;

  for (sy = 0; sy < i->y; sy += s)
    {
      uchar *scptr, *dcptr;
      int dx, sx;

      scptr = srptr;
      dcptr = drptr;
      dx = sx = 0;

      for (sx = 0; sx < i->x; sx += s)
        {
          *(dcptr++) = *scptr;
          scptr += s;
          dx++;
        }
      memset (dcptr, MGN_WHITE, (i->x) - dx);


      drptr += i->x;
      srptr += s * (i->x);
      dy++;

    }

  for (; dy < i->y; dy++)
    {
      memset (drptr, MGN_WHITE, i->x);
      drptr += i->x;
    }

}

static int
image_is_safe_to_use_shm (Display * d)
{
  char *s = XDisplayString (d);

  if (s[0] == ':')
    return (1);


  fprintf (MOAN, "****XShm Extension found but since your $DISPLAY\n");
  fprintf (MOAN, "****variable doesn't start with : guessing that\n");
  fprintf (MOAN, "****you are running remote - if not fix image.c\n");

  return (0);

}


int
get_best_x_image_type (Display * d)
{

  if (XShmQueryExtension (d))
    {

      if (image_is_safe_to_use_shm (d))
#if defined(__alpha__) && defined(__linux__)
        fprintf (MOAN, "****on alpha arch blocking use of Shm\n");
      return (MGN_XIMAGE);
#else
        return (MGN_SHXIMAGE);
#endif

    }


  fprintf (MOAN, "****Cannot use Shared Memory Extension - this will\n");
  fprintf (MOAN, "****really slow up this program, consider fixing this\n");

  return (MGN_XIMAGE);

}
