/*
 * $Log: profile.c,v $
 * Revision 14.0  1998/07/31 19:46:47  root
 * \#makecheckin
 *
 * Revision 13.0  1998/07/31 19:46:30  root
 * \#makecheckin
 *
 * Revision 12.0  1998/07/31 09:58:26  root
 * \#makecheckin
 *
 * Revision 11.0  1998/05/02 23:03:44  root
 * \#makecheckin
 *
 * Revision 10.2  1998/04/24 20:58:37  root
 * \#makecheckin
 *
 * Revision 10.1  1997/10/22 18:35:50  root
 * \#makecheckin
 *
 * Revision 10.0  1997/10/22 18:33:55  root
 * \#makecheckin
 *
 * Revision 9.0  1997/10/22 18:32:48  root
 * \#makecheckin
 *
 * Revision 8.0  1997/10/22 18:32:00  root
 * \#makecheckin
 *
 * Revision 7.0  1997/10/22 18:31:42  root
 * \#makecheckin
 *
 */

static char rcsid[] = "$Id: profile.c,v 14.0 1998/07/31 19:46:47 root Exp root $";

#include "mgn.h"

#define PROFILE_WIDTH 500
#define PROFILE_HEIGHT 200

#define TEXT_LEN1 40
#define TEXT_LEN2 52

#define SEARCH_POINTS 20
#define TRIANGLE 3

static void 
clear_prof_window (Profile_private * p)
{
  XSetForeground (p->display, p->gc, MGN_BLACK);
  XFillRectangle (p->display, p->xid, p->gc, 0, 0, PROFILE_WIDTH, PROFILE_HEIGHT);
}


static void 
draw_prof_bar (Profile_private * p, float l)
{
  int x = (int) (l * (float) PROFILE_WIDTH);
  XSetForeground (p->display, p->gc, MGN_BLUE);
  XDrawLine (p->display, p->xid, p->gc, x, 0, x, PROFILE_HEIGHT);
}



static void 
draw_prof_line (Profile_private * p)
{
  float o, s, l;
  int i;
  float *ptr;
  XPoint points[PROFILE_POINTS];

  ptr = p->prof;

  i = PROFILE_POINTS;

  {
    float max = -1.0e15, min = +1.0e15;
    char buf[1024];

    while (i--)
      {
        if (*ptr > max)
          max = *ptr;
        if (*ptr < min)
          min = *ptr;
        ptr++;
      }


    sprintf (buf, "Range is %.2fm to %.2fm", min, max);
    xv_set (p->range, PANEL_VALUE, buf, 0);

    o = max;
    s = ((float) PROFILE_HEIGHT) / (min - max);
  }

  l = 0;

  for (i = 0; i < PROFILE_POINTS; ++i)
    {

      points[i].x = (int) (l * (float) PROFILE_WIDTH);
      points[i].y = (int) (s * ((p->prof[i]) - o));

      l += PROFILE_STEP;
    }


  XSetForeground (p->display, p->gc, MGN_RED);
  XDrawLines (p->display, p->xid, p->gc, points, PROFILE_POINTS, CoordModeOrigin);

}


static void 
show_instructions (Profile_private * p)
{
  char buf[1024];

  buf[0] = 0;

  if (!p)
    return;

  switch (p->state)
    {
    case PROFILE_NONE:
      xv_set (p->instructions, PANEL_VALUE, "Click start to start", 0);
      sprintf (buf, "the moon is made of cheese");
      xv_set (p->range, PANEL_VALUE, "(but not fish)", 0);
      clear_prof_window (p);
      break;
    case PROFILE_FIRSTPT:
      xv_set (p->instructions, PANEL_VALUE, "Click on first point", 0);
      sprintf (buf, "the moon is still made of cheese");
      break;
    case PROFILE_SECONDPT:
      xv_set (p->instructions, PANEL_VALUE, "Click on second point", 0);
      sprintf (buf, "Lat %.3f Lon %.3f -", p->start.lat, p->start.lon);
      break;
    case PROFILE_DISPLAYED:
      xv_set (p->instructions, PANEL_VALUE, "Done (Click on start to start)", 0);
      sprintf (buf, "Lat %.3f Lon %.3f - Lat %.3f Lon %.3f",
               p->start.lat, p->start.lon, p->end.lat, p->end.lon);
      break;
    }
  xv_set (p->where, PANEL_VALUE, buf, 0);

}

int
profile_event (View v, Event * e)
{
  Profile_private *p = v->profile_private;
  LL ll;
  IXY ixy;
  int ac;

  if (!p)
    return 0;

  switch (p->state)
    {
    case PROFILE_NONE:
    case PROFILE_DISPLAYED:
      return 0;
    }

  ac = event_action (e);

  if (ac == ACTION_SELECT)
    ac = MS_LEFT;

  if ((ac != MS_LEFT) || (!event_is_down (e)))
    return 0;

  ixy.x = event_x (e);
  ixy.y = event_y (e);

  ll = ixy_to_ll (v, ixy);

  switch (p->state)
    {
    case PROFILE_SECONDPT:
      p->end = ll;
      p->state = PROFILE_DISPLAYED;
      ext_view_repaint (v);
      break;
    case PROFILE_FIRSTPT:
      p->start = ll;
      p->state = PROFILE_SECONDPT;
      break;
    }

  show_instructions (p);

  return 1;

}


static void
dismiss_notify (Panel_item i)
{
  View v = (View) xv_get (i, XV_KEY_DATA, VIEW_KEY);
  Profile_private *p = v->profile_private;

  if (!p)
    return;

  xv_set (p->frame, XV_SHOW, FALSE, 0);
  p->on = 0;
  p->state = PROFILE_NONE;
  show_instructions (p);

}

static void
start_notify (Panel_item i)
{
  View v = (View) xv_get (i, XV_KEY_DATA, VIEW_KEY);
  Profile_private *p = v->profile_private;

  if (!p)
    return;

  p->state = PROFILE_FIRSTPT;
  show_instructions (p);

}

static void
abort_notify (Panel_item i)
{
  View v = (View) xv_get (i, XV_KEY_DATA, VIEW_KEY);
  Profile_private *p = v->profile_private;

  if (!p)
    return;

  p->state = PROFILE_NONE;
  show_instructions (p);

}

#define PROFILE_NAME "profile.sav"

static void save_notify(Panel_item pi)
{
  View v = (View) xv_get (pi, XV_KEY_DATA, VIEW_KEY);
  Profile_private *p = v->profile_private;

  float  l;
  int i;

  FILE *f;

  l = 0;


  f=fopen(PROFILE_NAME,"w");

  if (!f) {
	notice_prompt(p->panel,NULL,
		NOTICE_MESSAGE_STRINGS,"Cannot open file:",
			PROFILE_NAME,NULL,
		NOTICE_BUTTON_YES,"Abort",
		NULL);

	return;
  }

      fprintf(f, "#Profile from (lat,lon) (%f,%f) to (%f,%f)\n",
		p->start.lat,p->start.lon,
		p->end.lat,p->end.lon);

      fprintf(f, "# %d steps on great circle path\n",PROFILE_POINTS);
	fprintf(f,"# normalized dist on path, height km, lat,lon \n");
	fprintf(f,"#\n");


  for (i = 0; i < PROFILE_POINTS; ++i)
    {
	
      fprintf(f, "%08.3f %08.3f %08.3f %08.3f\n",l,p->prof[i],
		p->pos[i].lat,p->pos[i].lon);

      l += PROFILE_STEP;
    }

  fclose(f);

	notice_prompt(p->panel,NULL,
		NOTICE_MESSAGE_STRINGS,"Profile saved in file",
			PROFILE_NAME,NULL,
		NOTICE_BUTTON_YES,"Ok",
		NULL);


}


static void 
draw_triangle (View v, Alttile t, IXY * altpoints)
{
  IXY xy[TRIANGLE + 1];
  int i;

  for (i = 0; i < TRIANGLE; ++i)
    xy[i] = ll_to_ixy (v, t->data[altpoints[i].x][altpoints[i].y].pos);

  xy[TRIANGLE] = xy[0];


  view_draw_line (v, xy, TRIANGLE + 1);



}


static int 
all_points_share_x (IXY * altpoints, int n)
{
  int x = altpoints->x;

  n--;
  altpoints++;

  while (n--)
    {
      if (altpoints->x != x)
        return (0);
      altpoints++;
    }

  return 1;
}

static int 
all_points_share_y (IXY * altpoints, int n)
{
  int y = altpoints->y;

  n--;
  altpoints++;

  while (n--)
    {
      if (altpoints->y != y)
        return (0);
      altpoints++;
    }

  return 1;
}


static int 
choose_linearly_independant_points (IXY * altpoints, int n)
{
  int m = TRIANGLE;

  while (all_points_share_x (altpoints, m) || all_points_share_y (altpoints, m))
    {

      if (m == n)
        {
          /* Exausted possibilities give up */
          return (TRIANGLE);
        }

      m++;

    }

  if (m > TRIANGLE)
    {                           /*We can fix,swap in li coord. */
      IXY t = altpoints[TRIANGLE - 1];
      altpoints[TRIANGLE - 1] = altpoints[m - 1];
      altpoints[m - 1] = t;
    }


  return (m);
}


void 
profile_repaint (View v, Alttile t)
{
  Profile_private *p = v->profile_private;
  float l;
  GCPath path;
  LL pos;

  int n, m, i;
  IXY altpoints[SEARCH_POINTS]; /* cache of nearest points in tile */
  IXY xy[PROFILE_POINTS];       /* Line of profile on view */

  LL points[TRIANGLE];
  float heights[TRIANGLE];
  IXY last[TRIANGLE];


  if (p->state != PROFILE_DISPLAYED)
    return;
  if (!t)
    return;


  bzero (last, sizeof (IXY) * TRIANGLE);

  path = make_gcpath (p->start, p->end);


  clear_prof_window (p);



  view_set_color (v, MGN_YELLOW);

  l = 0;
  for (n = 0; n < PROFILE_POINTS; ++n)
    {
      p->pos[n]=pos = get_point_on_gcpath (path, l);

      if ((n == 0) ||
      (!check_nearest_m_of_n_in_tile (t, pos, m, SEARCH_POINTS, altpoints)))
        {

          alt_find_nearest_n_in_tile (t, pos, SEARCH_POINTS, altpoints);
          m = choose_linearly_independant_points (altpoints, SEARCH_POINTS);

          for (i = 0; i < TRIANGLE; ++i)
            {
              points[i] = t->data[altpoints[i].x][altpoints[i].y].pos;
              heights[i] = t->data[altpoints[i].x][altpoints[i].y].rad;
            }

        }


      if (memcmp (last, altpoints, TRIANGLE * sizeof (IXY)))
        {
          draw_triangle (v, t, altpoints);
          draw_prof_bar (p, l);
          bcopy (altpoints, last, TRIANGLE * sizeof (IXY));
        }

      p->prof[n] = interpolate (points, heights, pos);

      xy[n] = ll_to_ixy (v, pos);
      l += PROFILE_STEP;
    }

  view_set_color (v, MGN_RED);
  view_draw_line (v, xy, PROFILE_POINTS);

  draw_prof_line (p);


}






static void
profile_notify (Menu m, Menu_item mi)
{
  View v = (View) xv_get (mi, XV_KEY_DATA, VIEW_KEY);
  Profile_private *p = v->profile_private;

  if (!p)
    return;

  xv_set (p->frame, XV_SHOW, TRUE, 0);
  p->on = 1;
  p->state = PROFILE_NONE;
  show_instructions (p);
}

void
profile_add_menu (View v, Menu m)
{

  Menu_item mi;


  mi = xv_create (0, MENUITEM,
                  MENU_STRING, "Profile",
                  MENU_NOTIFY_PROC, profile_notify,
                  XV_KEY_DATA, VIEW_KEY, v,
                  0);


  xv_set (m, MENU_APPEND_ITEM, mi, 0);

}

void
profile_private_destructor (View v)
{
  if (v->profile_private)
    {
      XFreeGC (v->profile_private->display, v->profile_private->gc);
      free (v->profile_private);
      v->profile_private = NULL;
    }

}

void
profile_private_init (View v)
{
  Profile_private *p;

  p = v->profile_private = (Profile_private *) malloc (sizeof (Profile_private));

  p->frame = xv_create (v->frame, FRAME,
                        XV_WIDTH, PROFILE_WIDTH,
                        FRAME_SHOW_LABEL, FALSE,
                        WIN_CMS, v->ctrl_cms,
                        XV_SHOW, FALSE,
                        0);


  p->panel = xv_create (p->frame, PANEL,
                        XV_WIDTH, PROFILE_WIDTH,
                        WIN_CMS, v->ctrl_cms,
                        0);


  xv_create (p->panel, PANEL_BUTTON,
             PANEL_LABEL_STRING, "Dismiss",
             PANEL_NOTIFY_PROC, dismiss_notify,
             XV_KEY_DATA, VIEW_KEY, v,
             0);



  xv_create (p->panel, PANEL_BUTTON,
             PANEL_LABEL_STRING, "Start",
             PANEL_NOTIFY_PROC, start_notify,
             XV_KEY_DATA, VIEW_KEY, v,
             0);


  xv_create (p->panel, PANEL_BUTTON,
             PANEL_LABEL_STRING, "Abort",
             PANEL_NOTIFY_PROC, abort_notify,
             XV_KEY_DATA, VIEW_KEY, v,
             0);

  xv_create (p->panel, PANEL_BUTTON,
             PANEL_LABEL_STRING, "Save",
             PANEL_NOTIFY_PROC, save_notify,
             XV_KEY_DATA, VIEW_KEY, v,
             0);


  p->instructions = xv_create (p->panel, PANEL_TEXT,
                               PANEL_LABEL_STRING, "Instructions:",
                               PANEL_READ_ONLY, TRUE,
                               PANEL_VALUE_DISPLAY_LENGTH, TEXT_LEN1,
                               XV_X, 0,
                               XV_Y, 30,
                               0);
  p->where = xv_create (p->panel, PANEL_TEXT,
                        PANEL_LABEL_STRING, "where:",
                        PANEL_READ_ONLY, TRUE,
                        PANEL_VALUE_DISPLAY_LENGTH, TEXT_LEN2,
                        XV_X, 0,
                        XV_Y, 60,
                        0);

  p->range = xv_create (p->panel, PANEL_TEXT,
                        PANEL_LABEL_STRING, "range:",
                        PANEL_READ_ONLY, TRUE,
                        PANEL_VALUE_DISPLAY_LENGTH, TEXT_LEN2,
                        XV_X, 0,
                        XV_Y, 90,
                        0);

  window_fit (p->panel);

  p->canvas = xv_create (p->frame, CANVAS,
                         WIN_BELOW, p->panel,
                         XV_X, 0,
                         XV_WIDTH, PROFILE_WIDTH,
                         XV_HEIGHT, PROFILE_HEIGHT,
                         WIN_CMS, v->cms,
                         CANVAS_RETAINED, TRUE,
                         0);


  window_fit (p->frame);

  p->display = (Display *) xv_get (p->canvas, XV_DISPLAY);

  p->xid = xv_get (canvas_paint_window (p->canvas), XV_XID);

  {
    XGCValues gcv;
    gcv.graphics_exposures = False;
    p->gc = XCreateGC (p->display, p->xid, GCGraphicsExposures, &gcv);
  }


}
