/*
 * $Log: tile.c,v $
 * Revision 14.0  1998/07/31 19:46:47  root
 * \#makecheckin
 *
 * Revision 13.0  1998/07/31 19:46:30  root
 * \#makecheckin
 *
 * Revision 12.0  1998/07/31 09:58:26  root
 * \#makecheckin
 *
 * Revision 11.0  1998/05/02 23:03:44  root
 * \#makecheckin
 *
 * Revision 10.0  1997/10/22 18:33:55  root
 * \#makecheckin
 *
 * Revision 6.0  1997/07/04  00:32:42  root
 * \#makecheckin
 *
 * Revision 5.0  1997/07/03 15:05:48  root
 * \#makecheckin
 *
 * Revision 4.0  1997/07/01 11:58:59  root
 * \#makecheckin
 *
 * Revision 3.0  1997/06/23 12:51:50  root
 * \#makecheckin
 *
 * Revision 2.0  1997/06/23 09:04:44  root
 * \#makecheckin
 *
 * Revision 1.10  1997/06/18 15:15:32  root
 * \#makecheckin
 *
 * Revision 1.9  1997/06/15 23:47:48  root
 * \#makecheckin
 *
 */

static char rcsid[] = "$Id: tile.c,v 14.0 1998/07/31 19:46:47 root Exp root $";


#include "mgn.h"

#define MAX_TILES 20            /* Maximum num of tiles to store */
#define MAX_BIG_TILES 10        /* Maximum num of image tiles store */

static int serial = 0;
static Tile_list list = NULL;


void
tile_destructor (Tile t)
{
  if (t->proj)
    free (t->proj);
  if (t->image)
    Obj_Delete (t->image);
  if (t->vicar)
    Obj_Delete (t->vicar);
  if (t->path)
    free (t->path);

  free (t);
}



Tile
create_tile (Create_image_args args, char *path)
{
  Tile t = CREATE_OBJ (Tile);
  Vicar v;
  Image i;


  t->vicar = open_vicar (path);

  if (!t->vicar)
    {
      free (t);
      t = NULL;
      return (t);
    }


  t->path = strdup (path);
  t->shrink = 1;

  args.x = t->vicar->nx;
  args.y = t->vicar->ny;

  if (!(t->vicar))
    {
      Obj_Delete (t);
      return (NULL);
    }


  t->image = create_image (args);

  if (!(t->image))
    {
      Obj_Delete (t);
      return (NULL);
    }

  t->vicar->data = t->image->data;

  if (t->vicar->data)
    read_vicar (t->vicar);

  t->proj = create_projection_from_vicar (t->vicar);
  t->stamp = serial++;

  return (t);
}


Tile
get_tile (Create_image_args args, char *path, int shrink)
{
  int n, bn;
  Tile current, least, bleast;
  int lowest = serial + 1;
  int blowest = serial + 1;

  if (!list)
    list = CREATE_LIST (Tile);


  n = bn = 0;

  current = list->head;

  while (current)
    {

      if ((strcmp (current->path, path) == 0) && (shrink == current->shrink) &&
          (args.type == current->image->type))
        return (current);

      if (current->stamp < lowest)
        {
          lowest = current->stamp;
          least = current;
        }
      n++;

      if (current->image->type != MGN_NO_IMAGE)
        {
          if (current->stamp < blowest)
            {
              blowest = current->stamp;
              bleast = current;
            }
          bn++;
        }

      current = current->next;
    }


  if (bn >= MAX_BIG_TILES)
    {
      Obj_Remove (list, bleast);
      Obj_Delete (bleast);
    }


  if (least != bleast)
    {
      if (n >= MAX_TILES)
        {
          Obj_Remove (list, least);
          Obj_Delete (least);
        }
    }

  current = create_tile (args, path);

  if (!current)
    return (current);


  if (shrink > 1)
    shrink_image (current->image, shrink);

  current->shrink = shrink;


  Obj_Insert (list, current, ObjInsertHead);

  return (current);

}



void
empty_tile_cache ()
{
  if (list)
    Obj_Delete (list);
  list = NULL;
}
