/*
 * $Log: util.c,v $
 * Revision 14.0  1998/07/31 19:46:47  root
 * \#makecheckin
 *
 * Revision 13.0  1998/07/31 19:46:30  root
 * \#makecheckin
 *
 * Revision 12.0  1998/07/31 09:58:26  root
 * \#makecheckin
 *
 * Revision 11.0  1998/05/02 23:03:44  root
 * \#makecheckin
 *
 * Revision 10.1  1997/10/22 18:35:49  root
 * \#makecheckin
 *
 * Revision 10.0  1997/10/22 18:33:55  root
 * \#makecheckin
 *
 * Revision 6.0  1997/07/04  00:32:42  root
 * \#makecheckin
 *
 * Revision 5.0  1997/07/03 15:05:48  root
 * \#makecheckin
 *
 * Revision 4.0  1997/07/01 11:58:59  root
 * \#makecheckin
 *
 * Revision 3.0  1997/06/23 12:51:50  root
 * \#makecheckin
 *
 * Revision 2.0  1997/06/23 09:04:44  root
 * \#makecheckin
 *
 * Revision 1.10  1997/06/18 15:15:32  root
 * \#makecheckin
 *
 * Revision 1.9  1997/06/15 23:47:48  root
 * \#makecheckin
 *
 */

static char rcsid[] = "$Id: util.c,v 14.0 1998/07/31 19:46:47 root Exp root $";

#include "mgn.h"


int
modish (int a, int b)
{
  while (a >= b)
    a -= b;
  while (a < 0)
    a += b;

  return (a);

}

float
fmodish (float a, int ib)
{
  float b = ib;

  while (a >= b)
    a -= b;
  while (a < 0)
    a += b;

  return (a);

}

char *
mgn_getenv (char *s)
{
  char *ret = getenv (s);

  if (!ret)
    {
      fprintf (MOAN, "getenv: failed to find environment var %s\n", s);
      fprintf (MOAN, "getenv: perhaps you ought to set it\n");
    }
  return (ret);

}

/* Create a matrix in x major form */


alt_rec **
create_ar_matrix (int x, int y)
{
  alt_rec *data = (alt_rec *) malloc (sizeof (alt_rec) * x * y);
  alt_rec **ptrs = (alt_rec **) malloc (sizeof (alt_rec *) * x);

  alt_rec *p1 = data;
  alt_rec **p2 = ptrs;

  while (x--)
    {
      *p2 = p1;
      p1 += y;
      p2++;
    }

  return (ptrs);
}


void
free_ar_matrix (alt_rec ** base)
{
  free (*base);
  free (base);
}

XYZ 
ll_to_xyz (LL ll)
{
  XYZ xyz;

  extern double cos (double);
  extern double sin (double);

  xyz.x = cos (ll.lon * THST) * cos (ll.lat * THST);
  xyz.y = sin (ll.lon * THST) * cos (ll.lat * THST);
  xyz.z = sin (ll.lat * THST);

  return (xyz);
}

LL 
xyz_to_ll (XYZ xyz)
{
  LL ll;
  float sq;
  extern double atan2 (double, double);
  extern double sqrt (double);

  sq = xyz.x * xyz.x;
  sq += xyz.y * xyz.y;
  sq = sqrt (sq);

  ll.lon = atan2 (xyz.y, xyz.x) / THST;
  ll.lat = atan2 (xyz.z, sq) / THST;

  return (ll);

}

GCPath 
make_gcpath (LL l1, LL l2)
{
  GCPath ret;

  ret.start = ll_to_xyz (l1);
  ret.delta = ll_to_xyz (l2);

  ret.delta.x -= ret.start.x;
  ret.delta.y -= ret.start.y;
  ret.delta.z -= ret.start.z;

  return (ret);
}

LL 
get_point_on_gcpath (GCPath p, float l)
{
  XYZ x;

  x = p.start;
  x.x += l * p.delta.x;
  x.y += l * p.delta.y;
  x.z += l * p.delta.z;

  return (xyz_to_ll (x));
}



float 
interpolate (LL pos[3], float h[3], LL now)
{
  float a1, a2, a3;
  float det;
  float x2, x3, y2, y3, h2, h3, xi, yi;
  float ans;

  x2 = pos[1].lon - pos[0].lon;
  x3 = pos[2].lon - pos[0].lon;
  xi = now.lon - pos[0].lon;

  LON_WRAP (x2);
  LON_WRAP (x3);
  LON_WRAP (xi);

  y2 = pos[1].lat - pos[0].lat;
  y3 = pos[2].lat - pos[0].lat;
  yi = now.lat - pos[0].lat;


  h2 = h[1] - h[0];
  h3 = h[2] - h[0];

  det = (x2 * (y3)) + (x3 * (-y2));
  a1 = (h2 * (y3)) + (h3 * (-y2));
  a2 = (h2 * (-x3)) + (h3 * (x2));

  a1 = a1 / det;
  a2 = a2 / det;

  ans = (a1 * xi) + (a2 * yi);
  ans = ans + h[0];

  return (ans);
}
