/*
 * $Log: view.c,v $
 * Revision 14.0  1998/07/31 19:46:47  root
 * \#makecheckin
 *
 * Revision 13.0  1998/07/31 19:46:30  root
 * \#makecheckin
 *
 * Revision 12.0  1998/07/31 09:58:26  root
 * \#makecheckin
 *
 * Revision 11.0  1998/05/02 23:03:44  root
 * \#makecheckin
 *
 * Revision 10.1  1997/10/22 18:35:48  root
 * \#makecheckin
 *
 * Revision 10.0  1997/10/22 18:33:55  root
 * \#makecheckin
 *
 * Revision 6.0  1997/07/04  00:32:42  root
 * \#makecheckin
 *
 * Revision 5.1  1997/07/04 00:31:02  root
 * \#makecheckin
 *
 * Revision 5.0  1997/07/03 15:05:48  root
 * \#makecheckin
 *
 * Revision 4.0  1997/07/01 11:58:59  root
 * \#makecheckin
 *
 * Revision 3.1  1997/07/01 11:58:45  root
 * \#makecheckin
 *
 * Revision 3.0  1997/06/23 12:51:50  root
 * \#makecheckin
 *
 * Revision 2.0  1997/06/23 09:04:44  root
 * \#makecheckin
 *
 * Revision 1.18  1997/06/23 08:53:03  root
 * \#makecheckin
 *
 * Revision 1.17  1997/06/18 15:15:32  root
 * \#makecheckin
 *
 * Revision 1.16  1997/06/15 23:47:48  root
 * \#makecheckin
 *
 */

static char rcsid[] = "$Id: view.c,v 14.0 1998/07/31 19:46:47 root Exp root $";


#include "mgn.h"

#define DEFAULT_WIN_WIDTH 512
#define DEFAULT_WIN_HEIGHT 512



void
view_destructor (View v)
{

  /* Hooks for private destructors go here */
  altview_private_destructor (v);
  return_private_destructor (v);
  info_private_destructor (v);
  whereami_private_destructor (v);
  profile_private_destructor (v);


  if (v->midr)
    Obj_Delete (v->midr);
  if (v->repaint_hook)
    Obj_Delete (v->repaint_hook);
  xv_destroy (v->cms);
  xv_destroy (v->menu);
  xv_destroy (v->ctrl_cms);
  XFreeGC (v->display, v->gc);

  free (v->colors);

  free (v);
}

void
info_at (View v, char *buf)
{
  IXY ixy;
  LL ll;

  ixy.x = v->wx >> 1;
  ixy.y = v->wy >> 1;

  ll = ixy_to_ll (v, ixy);

  sprintf (buf, "Lat %.3f Lon %.3f", ll.lat, ll.lon);

}

void
info_midr_name (View v, char *buf)
{
  char *c;
  if (!v->midr)
    return;

  c = index (v->midr->path, '/');
  if (c)
    {
      c++;
    }
  else
    {
      c = v->midr->path;
    }

  strcpy (buf, c);
}

void
info_midr_path (View v, char *buf)
{
  if (!v->midr)
    return;
  strcpy (buf, v->midr->path);

}


static void
draw_browsebox (View v)
{
  int x1, x2, y1, y2;
  x1 = (v->x >> 3) / v->shrink;
  y1 = (v->y >> 3) / v->shrink;

  x2 = x1 + ((v->wx >> 3) / v->shrink);
  y2 = y1 + ((v->wy >> 3) / v->shrink);


  XSetFunction (v->display, v->gc, GXinvert);


  XDrawLine (v->display, v->window, v->gc, x1, y1, x2, y1);
  XDrawLine (v->display, v->window, v->gc, x2, y1, x2, y2);
  XDrawLine (v->display, v->window, v->gc, x2, y2, x1, y2);
  XDrawLine (v->display, v->window, v->gc, x1, y2, x1, y1);
}

void
view_register_repaint_proc (View v, void (*proc) (View))
{
  Repaint_Procedure r;

  if (!v->repaint_hook)
    v->repaint_hook = CREATE_LIST (Repaint_Procedure);

  r = CREATE_OBJ (Repaint_Procedure);

  r->repaint_proc = proc;

  Obj_Insert (v->repaint_hook, r, ObjInsertTail);

}


static void
view_repaint (Canvas c)
{
  View v = (View) xv_get (c, XV_KEY_DATA, VIEW_KEY);
  MXY mxy;

  v->wx = (int) xv_get (c, WIN_WIDTH, 0);
  v->wy = (int) xv_get (c, WIN_HEIGHT, 0);

  {
    float s1, s2;

    s1 = ((float) v->midr->max_x) / (float) v->wx;
    s1 *= 0.125;
    s1 = ceil (s1);
    s2 = ((float) v->midr->max_y) / (float) v->wy;
    s2 *= 0.125;
    s2 = ceil (s2);


    if (s2 > s1)
      s1 = s2;

    v->shrink = (int) s1;
  }

  switch (v->mode)
    {
    case V_NORMAL:
      mxy.m = v->midr;
      mxy.x = v->x;
      mxy.y = v->y;

      v->midr->funcs.display_chunk (v->display, v->window,
                                    v->gc, mxy, v->wx, v->wy);
      break;
    case V_BROWSING:
      {
        XSetForeground (v->display, v->gc, MGN_WHITE);
        XSetFunction (v->display, v->gc, GXcopy);

        XFillRectangle (v->display, v->window, v->gc, 0, 0, v->wx, v->wy);


        display_browse_tile (v->display, v->window, v->gc, v->midr, v->shrink);

      }
      break;
    }

  grid_repaint (v);
  alt_repaint (v);
  return_repaint (v);
  info_repaint (v);


  if (v->repaint_hook)
    {
      Repaint_Procedure r;

      r = v->repaint_hook->head;

      while (r)
        {

          r->repaint_proc (v);

          r = r->next;
        }


    }



  v->xor = 0;


}


void
ext_view_repaint (View v)
{
  view_repaint (v->canvas);
}

static struct mgn_cstruct
{
  int i;
  Xv_singlecolor c;
}
mgn_colors[] =
{
  MGN_BLACK,
  {
    0, 0, 0
  }
  ,
    MGN_WHITE,
  {
    255, 255, 255
  }
  ,
    MGN_BLUE,
  {
    0, 0, 255
  }
  ,
    MGN_GREEN,
  {
    0, 255, 0
  }
  ,
    MGN_RED,
  {
    255, 0, 0
  }
  ,
    MGN_YELLOW,
  {
    255, 255, 0
  }
  ,
};

#define MGN_N_COLORS (sizeof(mgn_colors)/sizeof(struct mgn_cstruct))

static void
adjust_contrast (View v, int ex, int ey)
{
  int x, y, i;
  float fact;

  ex <<= 8;
  ex = ex / v->wx;
  ey <<= 8;
  ey = ey / v->wy;

  if (ex > ey)
    {
      x = ex;
      ex = ey;
      ey = x;
    }

  if (ex > 255)
    ex = 255;
  if (ey > 255)
    ey = 255;
  if (ex < 0)
    ex = 0;
  if (ey < 0)
    ey = 0;

  fact = 255. / ((float) (ey - ex));

  for (i = 0; i < ex; ++i)
    v->colors[i].red = v->colors[i].green = v->colors[i].blue = 0;

  for (i = ex; i <= ey; ++i)
    {
      x = (int) ((float) (i - ex) * fact);
      v->colors[i].red = v->colors[i].green = v->colors[i].blue = x;
    }

  for (i = ey; i < 256; ++i)
    v->colors[i].red = v->colors[i].green = v->colors[i].blue = 255;


  for (i = 0; i < MGN_N_COLORS; ++i)
    {
      v->colors[mgn_colors[i].i] = mgn_colors[i].c;
    }



  xv_set (v->cms, CMS_COLORS, v->colors, 0);

  /* Make the menu text readable */
  i = 255;
  v->colors[i].red = v->colors[i].green = v->colors[i].blue = 0;

  xv_set (v->ctrl_cms, CMS_COLORS, v->colors + CMS_CONTROL_COLORS, 0);


}


static void
view_event (Canvas canvas, Event * event)
{
  int ex, ey;
  int ac;
  View v = (View) xv_get (canvas, XV_KEY_DATA, VIEW_KEY);

  ac = event_action (event);
  ex = event_x (event);
  ey = event_y (event);

  if ((xv_get (v->canvas, WIN_WIDTH, 0) != v->wx) ||
      (xv_get (v->canvas, WIN_HEIGHT, 0) != v->wy))
    {
      invalidate_altimetry_cache (v);
      ext_view_repaint (v);
    }

  if (profile_event (v, event))
    return;

  whereami_event (v, event);

  if (v->using_ctrl_cms)
    {
      v->using_ctrl_cms = 0;
      xv_set (canvas, WIN_CMS, v->cms, 0);
    }

  if ((event_id (event) == MS_RIGHT) && event_is_down (event))
    {
      xv_set (canvas, WIN_CMS, v->ctrl_cms, 0);
      v->using_ctrl_cms++;
      v->menu_pos.x = ex;
      v->menu_pos.y = ey;
      menu_show (v->menu, canvas, event, NULL);
    }

  if (ac == ACTION_SELECT)
    ac = MS_LEFT;
  if (ac == ACTION_ADJUST)
    ac = MS_MIDDLE;
  if (ac == ACTION_MENU)
    ac = MS_RIGHT;

  if ((int) event_middle_is_down (event) != 0)
    {
      adjust_contrast (v, ex, ey);
    }



  switch (v->mode)
    {
    case V_BROWSING:




      if (v->xor)
        draw_browsebox (v);
      v->x = (ex * 8 * v->shrink) - (v->wx >> 1);
      v->y = (ey * 8 * v->shrink) - (v->wy >> 1);


      draw_browsebox (v);
      v->xor = 1;


      switch (ac)
        {
        case LOC_WINEXIT:
          draw_browsebox (v);
          v->xor = 0;
          break;

        case MS_LEFT:
          if (event_is_down (event))
            {
              v->mode = V_NORMAL;
              invalidate_altimetry_cache (v);
              ext_view_repaint (v);
              return;
            }
          break;

        }
      break;

    case V_NORMAL:
      switch (ac)
        {
        case MS_LEFT:
          if (event_is_down (event))
            {
              v->mode = V_BROWSING;
              invalidate_altimetry_cache (v);
              ext_view_repaint (v);
              ex = ((v->x + (v->wx >> 1)) >> 3) / v->shrink;
              ey = ((v->y + (v->wy >> 1)) >> 3) / v->shrink;
              xv_set (v->canvas, WIN_MOUSE_XY, ex, ey, 0);
              return;
            }
          break;

        }
    }



}

static Notify_value
destroy_view (client, status)
     Notify_client client;
     Destroy_status status;
{
  Frame frame = (Frame) client;
  View v = (View) xv_get (frame, XV_KEY_DATA, VIEW_KEY);

  switch (status)
    {
    case DESTROY_CLEANUP:
      fprintf (MOAN, "Destroying view at %08x\n", v);
      if (v)
        Obj_Delete (v);
      xv_set (frame, XV_KEY_DATA, VIEW_KEY, NULL, NULL);
      return notify_next_destroy_func (client, status);
      break;
    }

  return NOTIFY_DONE;
}


XY
ll_to_xy (View v, LL ll)
{
  XY xy =
  {0, 0};
  switch (v->mode)
    {
    case V_NORMAL:
      xy = v->midr->funcs.ll_to_xy (v->midr, ll);
      break;
    case V_BROWSING:
      if (v->midr->browse_proj)
        {
          xy = v->midr->browse_proj->funcs.ll_to_xy (v->midr->browse_proj, ll);
        }
      break;
    }
  return (xy);

}

LL
xy_to_ll (View v, XY xy)
{
  LL ll =
  {0, 0};

  switch (v->mode)
    {
    case V_NORMAL:
      ll = v->midr->funcs.xy_to_ll (v->midr, xy);
      break;
    case V_BROWSING:

      if (v->midr->browse_proj)
        {
          ll = v->midr->browse_proj->funcs.xy_to_ll (v->midr->browse_proj, xy);
        }
      break;
    }

  return (ll);

}

IXY
xy_to_ixy (View v, XY xy)
{
  IXY ixy =
  {0, 0};

  switch (v->mode)
    {
    case V_NORMAL:
      ixy.x = (int) xy.x;
      ixy.y = (int) xy.y;
      ixy.x -= v->x;
      ixy.y -= v->y;
      break;
    case V_BROWSING:
      xy.x = xy.x / (float) v->shrink;
      xy.y = xy.y / (float) v->shrink;
      ixy.x = (int) xy.x;
      ixy.y = (int) xy.y;
      break;
    }
  return (ixy);


}

XY
ixy_to_xy (View v, IXY ixy)
{
  XY xy =
  {0, 0};

  switch (v->mode)
    {
    case V_NORMAL:
      xy.x = (float) ixy.x;
      xy.y = (float) ixy.y;
      xy.x += (float) v->x;
      xy.y += (float) v->y;
      break;
    case V_BROWSING:
      xy.x = (float) ixy.x;
      xy.y = (float) ixy.y;
      xy.y *= (float) v->shrink;
      xy.x *= (float) v->shrink;
      break;
    }
  return (xy);
}

IXY
ll_to_ixy (View v, LL ll)
{
  XY xy;
  xy = ll_to_xy (v, ll);
  return (xy_to_ixy (v, xy));
}

LL
ixy_to_ll (View v, IXY ixy)
{
  XY xy;
  xy = ixy_to_xy (v, ixy);
  return (xy_to_ll (v, xy));
}

static void
view_menu_quit (Menu menu, Menu_item item)
{
  View v = (View) xv_get (menu, XV_KEY_DATA, VIEW_KEY);
  exit (0);
}





View
create_view (char *path)
{
  XGCValues gcv;
  View v;
  Cms ctrl;



  {
    Midr m;

    m = open_midr (path);

    if (!m)
      return (NULL);

    v = CREATE_OBJ (View);

    v->midr = m;
  }

  v->colors = (Xv_singlecolor *) malloc (256 * sizeof (Xv_singlecolor));
  {
    int i;

    for (i = 0; i < 256; ++i)
      {
        v->colors[i].red = i;
        v->colors[i].green = i;
        v->colors[i].blue = i;
      }
  }




  v->cms = xv_create (0, CMS,
                      CMS_SIZE, 256,
                      CMS_TYPE, XV_DYNAMIC_CMS,
                      CMS_COLORS, v->colors,
                      NULL);

  v->colors[255].red = v->colors[255].green = v->colors[255].blue = 0;

  v->ctrl_cms = xv_create (0, CMS,
                           CMS_SIZE, 256,
                           CMS_TYPE, XV_DYNAMIC_CMS,
                           CMS_COLORS, v->colors + CMS_CONTROL_COLORS,
                           CMS_CONTROL_CMS, TRUE,
                           NULL);
  v->frame = xv_create (0, FRAME, XV_KEY_DATA, VIEW_KEY, v,
                        FRAME_SHOW_LABEL, FALSE,
                        WIN_CMS, v->ctrl_cms,
                        0);

  v->using_ctrl_cms++;
  v->canvas = window_create (v->frame, CANVAS,
                             WIN_CMS, v->ctrl_cms,
                             XV_X, 0,
                             XV_Y, 0,
                             XV_WIDTH, DEFAULT_WIN_WIDTH,
                             XV_HEIGHT, DEFAULT_WIN_HEIGHT,
                             CANVAS_RETAINED, FALSE,
                             CANVAS_REPAINT_PROC, view_repaint,
                             CANVAS_AUTO_SHRINK, TRUE,
                             CANVAS_AUTO_EXPAND, TRUE,
                             XV_KEY_DATA, VIEW_KEY, v,
                             0);

  xv_set (canvas_paint_window (v->canvas),
          XV_KEY_DATA, VIEW_KEY, v,
          XV_VISUAL_CLASS, PseudoColor,
          WIN_EVENT_PROC, view_event,
          WIN_CONSUME_EVENTS,
          KBD_DONE, KBD_USE, LOC_DRAG, LOC_MOVE, WIN_MOUSE_BUTTONS,
          LOC_WINEXIT,
          NULL,
          NULL);


  v->menu = xv_create (0, MENU,
                       MGN_PIN (v, "View")
                       MENU_TITLE_ITEM, "Mgnview",
                       XV_KEY_DATA, VIEW_KEY, v,
                       NULL);

  /* Hooks to the base menu go here */
  /* any per view variables for base menu parameters */
  /* should be stored using KEYS defined in keys.h */
  /* in the v->frame object */

  motion_base_menu (v);
  grid_base_menu (v);
  alt_base_menu (v);
  info_base_menu (v);
  whereami_base_menu (v);


  xv_set (v->menu, MENU_ITEM,
          MENU_STRING, "Quit",
          MENU_NOTIFY_PROC, view_menu_quit, 0,
          0);



  v->display = (Display *) xv_get (v->canvas, XV_DISPLAY);
  v->window = xv_get (canvas_paint_window (v->canvas), XV_XID);

  gcv.graphics_exposures = False;
  gcv.foreground = 255;
  v->gc = XCreateGC (v->display, v->window, GCGraphicsExposures, &gcv);

  window_fit (v->frame);



  notify_interpose_destroy_func (v->frame, destroy_view);


  if (v->midr->browse)
    v->mode = V_BROWSING;
  v->shrink = 1024 / DEFAULT_WIN_WIDTH;

  /* Hooks for private inits go here */
  altview_private_init (v);
  return_private_init (v);
  info_private_init (v);
  whereami_private_init (v);
  profile_private_init (v);

  v->wx = DEFAULT_WIN_WIDTH;
  v->wy = DEFAULT_WIN_HEIGHT;

  adjust_contrast (v, 0, v->wy);



  return (v);

}
