/* 
 * $Log: cdrom.c,v $
 * Revision 1.7  1998/07/31 16:59:48  root
 * makecheckin
 *
 * Revision 1.6  1997/06/23 08:53:13  root
 * makecheckin
 *
 * Revision 1.5  1997/06/15 23:36:54  root
 * makecheckin
 *
 */

static char rcsid[]="$Id: cdrom.c,v 1.7 1998/07/31 16:59:48 root Exp root $";

#include "vfs.h"
#include <ftw.h>
#define CDROM_FTW_DEPTH 8


static void
get_cdrom_id_from_path (char *path, char *id)
{
  char buf[1024];
  FILE *f;

  id[0] = 0;

  strcpy (buf, path);
  strcat (buf, "/voldesc.sfd");

  f = fopen (buf, "r");
  if (!f)
    return;

  while (!feof (f))
    {
      read_line_from_file (f, buf);
      if (strstr (buf, "VOLUME_ID"))
        {
          char *p;
          p = index (buf, '=');

          if (!p)
            {
              fclose (f);
              return;
            }
          p++;


          strip (p);
          strcpy (id, p);

        }


    }

 fclose(f);

}


static int
vfs_scan_cdroms (char *id, char *path)
{
  char *p = vfs_getenv ("MGNVIEW_CDROMS");
  char buf[1024], b2[1024];
  char rid[1024];

  path[0] = 0;

  if (!p)
    return (0);
  strcpy (buf, p);

  p = buf;

  while (p)
    {
      char *p2;

      strcpy (b2, p);
      p2 = index (b2, ':');
      if (p2)
        *p2 = 0;

      get_cdrom_id_from_path (b2, rid);

      if (strcmp (id, rid) == 0)
        {
          if (path)
            strcpy (path, b2);
          return 1;
        }

      strcat(b2,"/");
      strcat(b2,id);

      get_cdrom_id_from_path (b2, rid);

      if (strcmp (id, rid) == 0)
        {
          if (path)
            strcpy (path, b2);
          return 1;
        }






      p = index (p, ':');
      if (p)
        p++;
    }


  return (0);

}

int
vfs_check_attached_cdroms (char *id, char *path, char *ret)
{
  int i = vfs_scan_cdroms (id, ret);
  if (!i)
    return (i);

  strcat (ret, "/");
  strcat (ret, path);

  return (1);
}

static int cd_scan_skip;
static FILE *cd_scan_file;


static int
alt_scan_func (const char *file, struct stat *s, int flag)
{
  Volent e;
  int rt = REC_TYPE_VOLENT;

  bzero (&e, sizeof (Volent));

  if (flag != FTW_F)
    return (0);
  {

    strcpy (e.path, file + cd_scan_skip);
    strcpy (e.name, vfs_standardize_altimetry_name (e.path));

    if ((strncmp ("adf", e.name, 3) == 0) && (strstr (e.name, "lbl") == NULL))
      {

        printf ("Name %s File %s\n", e.name, e.path);

        BO_I_MTOF (&rt,1);

        fwrite (&rt, sizeof (int), 1, cd_scan_file);
        fwrite (&e, sizeof (Volent), 1, cd_scan_file);



      }

  }


  return (0);
}


static void
vfs_scan_cdrom_for_altimetry (char *path)
{
  Volid v;
  bzero (&v, sizeof (Volid));

  v.type = VOL_TYPE_CDROM;

  get_cdrom_id_from_path (path, v.id);

  if (v.id[0] == 0)
    {
      fprintf (MOAN, "vfs: cannot find volume name of %s\n", path);
      return;
    }

  fprintf (MOAN, "vfs: %s contains %s\n", path, v.id);


  {
    char buf[1024];
    int rt;

    char *f = vfs_getenv ("MGNVIEW");
    if (!f)
      return;
    strcpy (buf, f);
    strcat (buf, "/vfs/altimetry/");
    strcat (buf, v.id);

    cd_scan_file = fopen (buf, "w");

    if (!cd_scan_file) {
	fprintf(MOAN,"vfs: fatal: couldn't open %s for writing\n",buf);
	fprintf(MOAN,"vfs: fatal: check ownership, path etc...\n");
	commit_suicide();
    }

    rt = REC_TYPE_VOLID;

    BO_I_MTOF (&rt,1);
    BO_I_MTOF (&v.type,1);

    fwrite (&rt, sizeof (int), 1, cd_scan_file);
    fwrite (&v, sizeof (Volid), 1, cd_scan_file);

  }

  cd_scan_skip = strlen (path) + 1;

  ftw (path, alt_scan_func, CDROM_FTW_DEPTH);

  fclose (cd_scan_file);


}


static void 
analyse_browse_image (const char *path,char *name)
{
  Vicar v;
  Projection *p;

  FILE *f = fopen (path, "r");

  if (!f)
    return;

  v=open_vicar_by_file(path,f);


  vfs_write_int(cd_scan_file,REC_TYPE_VICARHEAD);
  vfs_write_string(cd_scan_file,name);
  vfs_write_vicarheadrec(cd_scan_file,v);

  
  p=create_projection_from_vicar(v);

  vfs_write_int(cd_scan_file,REC_TYPE_PROJECTION);
  vfs_write_string(cd_scan_file,name);
  vfs_write_int(cd_scan_file,v->nx);
  vfs_write_int(cd_scan_file,v->ny);
  vfs_write_projection(cd_scan_file,p);

  free(p);

  Obj_Delete(v);


}




static int
midr_scan_func (const char *file, struct stat *s, int flag)
{
  Volent e;
  int rt = REC_TYPE_VOLENT;
  const char *end = rindex (file, '/');

  bzero (&e, sizeof (Volent));


  if (flag != FTW_F)
    return (0);
  if (!end)
    end = file;
  else
    end++;


  if (strcmp (end, "browse.img") == 0)
    {
      strcpy (e.path, file + cd_scan_skip);
      strcpy (e.name, vfs_standardize_midr_name (e.path));

      printf ("Name %s File %s\n", e.name, e.path);

      BO_I_MTOF (&rt,1);

      fwrite (&rt, sizeof (int), 1, cd_scan_file);
      fwrite (&e, sizeof (Volent), 1, cd_scan_file);

      analyse_browse_image (file, e.name);

    }


  return (0);
}


static void
vfs_scan_cdrom_for_midrs (char *path)
{
  Volid v;
  bzero (&v, sizeof (Volid));

  v.type = VOL_TYPE_CDROM;

  get_cdrom_id_from_path (path, v.id);

  if (v.id[0] == 0)
    {
      fprintf (MOAN, "vfs: cannot find volume name of %s\n", path);
      return;
    }

  fprintf (MOAN, "vfs: %s contains %s\n", path, v.id);


  {
    char buf[1024];
    int rt;

    char *f = vfs_getenv ("MGNVIEW");
    if (!f)
      return;
    strcpy (buf, f);
    strcat (buf, "/vfs/midr/");
    strcat (buf, v.id);

    cd_scan_file = fopen (buf, "w");

    if (!cd_scan_file) {
	fprintf(MOAN,"vfs: fatal: couldn't open %s for writing\n",buf);
	fprintf(MOAN,"vfs: fatal: check ownership, path etc...\n");
	commit_suicide();
    }


    rt = REC_TYPE_VOLID;

    BO_I_MTOF (&rt,1);
    BO_I_MTOF (&v.type,1);

    fwrite (&rt, sizeof (int), 1, cd_scan_file);
    fwrite (&v, sizeof (Volid), 1, cd_scan_file);

  }

  cd_scan_skip = strlen (path) + 1;

  ftw (path, midr_scan_func, CDROM_FTW_DEPTH);

  fclose (cd_scan_file);


}


void
vfs_scan_cdrom (char *path)
{
  vfs_scan_cdrom_for_altimetry (path);
  vfs_scan_cdrom_for_midrs (path);

}



