/*
 * $Log: projection.c,v $
 * Revision 1.1  1998/07/31 16:59:48  root
 * makecheckin
 *
 * Revision 12.0  1998/07/31 09:58:26  root
 * \#makecheckin
 *
 * Revision 11.0  1998/05/02 23:03:44  root
 * \#makecheckin
 *
 * Revision 10.0  1997/10/22 18:33:55  root
 * \#makecheckin
 *
 * Revision 6.0  1997/07/04  00:32:42  root
 * \#makecheckin
 *
 * Revision 5.0  1997/07/03 15:05:48  root
 * \#makecheckin
 *
 * Revision 4.0  1997/07/01 11:58:59  root
 * \#makecheckin
 *
 * Revision 3.0  1997/06/23 12:51:50  root
 * \#makecheckin
 *
 * Revision 2.0  1997/06/23 09:04:44  root
 * \#makecheckin
 *
 * Revision 1.9  1997/06/18 15:15:32  root
 * \#makecheckin
 *
 * Revision 1.8  1997/06/15 23:47:48  root
 * \#makecheckin
 *
 */

static char rcsid[] = "$Id: projection.c,v 1.1 1998/07/31 16:59:48 root Exp root $";

#include "vfs.h"

#ifndef THST
#define THST (M_PI/180.0)
#endif


extern double cos (double);

static XY
sin_ll_to_xy (Projection * p, LL ll)
{
  float f1;
  XY xy =
  {0, 0};


  f1 = ll.lon - (p->data.sin.projlong);

  while (f1 < -180)
    f1 = f1 + 360;
  while (f1 >= 180)
    f1 = f1 - 360;

  f1 *= (p->data.sin.a) * cos (ll.lat * THST);

  xy.x = (p->data.sin.projsamp) + f1;
  xy.y = (p->data.sin.a) * ((p->data.sin.lat_ul) - ll.lat);

  return (xy);
}

static LL
sin_xy_to_ll (Projection * p, XY xy)
{
  float f1;
  LL ll =
  {0, 0};

  ll.lat = p->data.sin.lat_ul - (xy.y / (p->data.sin.a));

  f1 = xy.x - p->data.sin.projsamp;
  f1 *= 1.0 / ((p->data.sin.a) * cos (ll.lat * THST));

  while (f1 < -180)
    f1 = f1 + 360;
  while (f1 >= 180)
    f1 = f1 - 360;


  ll.lon = p->data.sin.projlong + f1;

  return (ll);
}

static struct proj_funcs sin_funcs =
{sin_ll_to_xy, sin_xy_to_ll};

static Projection *
sin_init_from_vicar (Vicar v)
{
  Projection *ret = malloc (sizeof (Projection));
  float y;

  bzero (ret, sizeof (Projection));

  ret->funcs = sin_funcs;

  vicar_get_float (v, "LAT_UL", &ret->data.sin.lat_ul);
  vicar_get_float (v, "LAT_LL", &ret->data.sin.lat_ll);
  vicar_get_float (v, "PROJ_LON", &ret->data.sin.projlong);
  vicar_get_float (v, "PROJSAMP", &ret->data.sin.projsamp);
  vicar_get_float (v, "NL", &y);

  /* HORRIBLE KLUDGE BUT IT'S JPLS FAULT */
  if (strcmp (v->name, "browse.img") == 0)
    {
      ret->data.sin.projsamp *= 0.125;
    }


  ret->data.sin.a = y / (ret->data.sin.lat_ul - ret->data.sin.lat_ll);

  ret->type = PROJ_SINUSOIDAL;

  ret->center_lon = ret->data.sin.projlong;

  return (ret);
}


static XY
none_ll_to_xy (Projection * p, LL ll)
{
  XY xy;
  xy.x = 0;
  xy.y = 0;
  fprintf (MOAN, "projection: transforming from unknown projection.\n");
  return (xy);
}

static LL
none_xy_to_ll (Projection * p, XY xy)
{
  LL ll;
  ll.lat = 0;
  ll.lon = 0;
  fprintf (MOAN, "projection: transforming to unknown projection.\n");
  return (ll);
}

static struct proj_funcs none_funcs =
{none_ll_to_xy, none_xy_to_ll};

static Projection *
none_init_from_vicar (Vicar v)
{
  Projection *ret = malloc (sizeof (Projection));
  float y;

  bzero (ret, sizeof (Projection));

  ret->funcs = none_funcs;

  ret->type = PROJ_NONE;

  return(ret);
}


static XY
lin_ll_to_xy (Projection * p, LL ll)
{


}

static LL
lin_xy_to_ll (Projection * p, XY ll)
{



}

static struct proj_funcs lin_funcs =
{lin_ll_to_xy, lin_xy_to_ll};

static void
lin_init_from_vicar (Vicar v)
{

}



Projection *
create_projection_from_vicar (Vicar v)
{
  char *s;

  if (!v)
    return (NULL);

  s = vicar_get_str (v, "MAP_PROJ");
  if (!s)
    return (none_init_from_vicar (v));


  if (strcmp (s, "SINUSOIDAL") == 0)
    {
      free (s);
      return (sin_init_from_vicar (v));
    }

  free (s);

  return (none_init_from_vicar (v));
}

vfs_write_projection(FILE *file,Projection *p)
{

vfs_write_int(file,p->type);

switch(p->type) {
case PROJ_NONE:
	return;
case PROJ_SINUSOIDAL:

vfs_write_float(file,p->data.sin.projsamp);
vfs_write_float(file,p->data.sin.projlong);
vfs_write_float(file,p->data.sin.lat_ul);
vfs_write_float(file,p->data.sin.lat_ll);
vfs_write_float(file,p->data.sin.a);

break;

default:
	fprintf(MOAN,"vfs_write_projection: fatal: unknown projection type %d\n",p->type);
	commit_suicide();
}

vfs_write_float(file,p->center_lon);

}

void vfs_skip_projection(FILE *file)
{

int type;

type=vfs_read_int(file);


switch(type) {
case PROJ_NONE:
	return;
case PROJ_SINUSOIDAL:
	fseek(file,5*sizeof(float),SEEK_CUR);
	break;

default:
	fprintf(MOAN,"vfs_skip_projection: fatal: unknown projection type %d\n",type);
	commit_suicide();
}

vfs_skip_float(file);

}

Projection * vfs_read_projection(FILE *file)
{
Projection *p;


p=(Projection *) malloc(sizeof(Projection));


p->type=vfs_read_int(file);

switch(p->type) {
case PROJ_NONE:
p->funcs=none_funcs;
	return(p);
case PROJ_SINUSOIDAL:
p->data.sin.projsamp=vfs_read_float(file);
p->data.sin.projlong=vfs_read_float(file);
p->data.sin.lat_ul=vfs_read_float(file);
p->data.sin.lat_ll=vfs_read_float(file);
p->data.sin.a=vfs_read_float(file);
p->funcs=sin_funcs;

	break;

default:
	fprintf(MOAN,"vfs_read_projection: fatal: unknown projection type %d\n",p->type);
	commit_suicide();
}

p->center_lon=vfs_read_float(file);

return(p);

}

