/* 
 * $Log: vfs.c,v $
 * Revision 1.8  1998/07/31 16:59:48  root
 * makecheckin
 *
 * Revision 1.7  1997/06/23 08:53:13  root
 * makecheckin
 *
 * Revision 1.6  1997/06/15 23:36:54  root
 * makecheckin
 *
 */

static char rcsid[]="$Id: vfs.c,v 1.8 1998/07/31 16:59:48 root Exp root $";

#include "vfs.h"

static int serial = 1;

char *vfs_vol_type[VOL_TYPE_MAX]={
	"Reserved",
	"New Tape",
	"CDROM   ",
	"Old Tape",
	"Mount   "};

typedef struct
  {
    char type[VOL_MAX_LEN];
    char path[VOL_MAX_LEN];
    int last;
  }
Cache_ent;

static Cache_ent *cache = NULL;


void vfsanswer_destructor(VFSAnswer a)
{
if (a->path) free(a->path);
free(a);
}

static void
vfs_init_cache ()
{

  if (cache)
    return;

  cache = (Cache_ent *) malloc (sizeof (Cache_ent) * VFS_CACHE_SIZE);
  bzero (cache, sizeof (Cache_ent) * VFS_CACHE_SIZE);

  return;

}

static void
vfs_add_to_cache (char *type, char *path)
{
  int least = serial + 1;
  int ileast = 0;
  int i;

  vfs_init_cache ();

  for (i = 0; i < VFS_CACHE_SIZE; ++i)
    {

      if (strcmp (path, cache[i].path) == 0)
        {
          cache[i].last = serial++;
          return;
        }

      if (cache[i].last < least)
        {
          least = cache[i].last;
          ileast = i;
        }

    }

  cache[ileast].last = serial++;
  strcpy (cache[ileast].type, type);
  strcpy (cache[ileast].path, path);

  return;
}




static int
vfs_get_rectype (FILE * f)
{
  int i;

  if (fread (&i, sizeof (int), 1, f) > 0)
    {

      BO_I_FTOM (&i,1);

      return (i);
    }
  else
    return (0);

}

static char * vfs_go_find(VFSAnswer a)
{
char ret[1024];

              switch (a->v.type)
                {
                case VOL_TYPE_CDROM:
                  if (vfs_check_attached_cdroms (a->v.id, a->path, ret))
                    {
  			fprintf (MOAN, "vfs_cdrom: found %s as %s\n", a->path, ret);
			return(strdup(ret));

                    }
                  else
                    {
                      fprintf (MOAN, "vfs: **** found %s on CDROM %s but not"
                               " mounted\n", a->path, a->v.id);

                    }


                  break;
                default:
                  fprintf (MOAN, "vfs: found %s on volume %s but unknown"
                           " medium type %d\n", a->path, a->v.id, a->v.type);

                }
		      return(NULL);

}

static char *vfs_go_find_one_in_list(VFSAnswer_list al)
{
VFSAnswer a;
char *r;

a=al->head;

while (a)
{

if (r=vfs_go_find(a)) return(r);

a=a->next;
}

return(NULL);
}

static void
vfs_scan_vfs_file (char *vfsname, Question q,VFSAnswer_list al)
{
  FILE *vfs = fopen (vfsname, "r");
  Volid v;
  Volent e;
  Projection *p;
  char *n;
  VFSAnswer a;

  int len;


  int r;

  if (!vfs)
    return;

  if (!al) return;

  if (vfs_get_rectype (vfs) != REC_TYPE_VOLID)
    {
      fprintf (MOAN, "vfs: file %s has wrong header\n", vfsname);
      fclose (vfs);
      return;
    }

  fread (&v, sizeof (Volid), 1, vfs);

  BO_I_FTOM (&v.type,1);

  while (r = vfs_get_rectype (vfs))
    {
      switch (r)
        {
        case REC_TYPE_VOLID:
          fprintf (MOAN, "vfs: file %s sequencing error\n", vfsname);
          fclose (vfs);
          return ;
          break;
        case REC_TYPE_PROJECTION:

	  switch (q.qtype) {
	  case VFS_Q_CONTAINS:
	{
	int nx,ny;
	XY xy;


		n=vfs_read_string(vfs);
		nx=vfs_read_int(vfs);
		ny=vfs_read_int(vfs);
		p=vfs_read_projection(vfs);

		xy=(p->funcs.ll_to_xy)(p,q.qdata.contains.ll);
		
		if ((xy.x>0) && (xy.x<nx) && (xy.y>0) &&(xy.y<ny)) {
		a=CREATE_OBJ(VFSAnswer);
		a->v=v;
		a->path=strdup(n);

		Obj_Insert(al,a,ObjInsertTail);

		}
		
		free(p);
		free(n);
	

	}

		
	
		break;
	  default:
		vfs_skip_string(vfs);
		vfs_skip_int(vfs);
		vfs_skip_int(vfs);
		vfs_skip_projection(vfs);
		break;
	  }

	  
	  break;
        case REC_TYPE_VICARHEAD:
	  switch (q.qtype) {
	  default:
		vfs_skip_string(vfs);
		vfs_skip_vicarheadrec(vfs);
		break;
	  }
          break;
        case REC_TYPE_VOLENT:
          fread (&e, sizeof (Volent), 1, vfs);

  	  switch (q.qtype) {
	  case VFS_Q_PATH:

          if (strcmp (e.name, q.qdata.path) == 0)
            {
		a=CREATE_OBJ(VFSAnswer);
		a->path=strdup(e.path);
		a->v=v;

		Obj_Insert(al,a,ObjInsertTail);
            }
	  }

          break;
        default:
          fprintf (MOAN, "vfs: file %s sequencing error\n", vfsname);
          fclose (vfs);
          return ;
          break;

        }
    }

  fclose(vfs);


  return ;

}



static char *
vfs_one_answer_question (Question q, char *type)
{
  char buf[1024];
  DIR *d;
  struct dirent *ent;
  VFSAnswer_list al;




  vfs_init_cache ();


  {
    int i;
    char *r;

    for (i = 0; i < VFS_CACHE_SIZE; ++i)
      {
        if (strcmp (cache[i].type, type) == 0)
          {

	    al=CREATE_LIST(VFSAnswer);
            vfs_scan_vfs_file (cache[i].path, q,al);

	    if (r=vfs_go_find_one_in_list(al))
              {
		Obj_Delete(al);

                cache[i].last = serial++;
                return (r);
              }
		Obj_Delete(al);




          }
      }
  }



  {
    char *f = vfs_getenv ("MGNVIEW");
    if (!f)
      return (f);
    strcpy (buf, f);
    strcat (buf, "/vfs/");
    strcat (buf, type);
  }

  d = opendir (buf);
  if (!d)
    {
      fprintf (MOAN, "vfs: no data for type %s\n", type);
      return (NULL);
    }

  while (ent = readdir (d))
    {
      char fname[1024], *r;
      strcpy (fname, buf);
      strcat (fname, "/");
      strcat (fname, ent->d_name);

      if (ent->d_name[0] != '.')
        {

           al=CREATE_LIST(VFSAnswer);
          vfs_scan_vfs_file (fname, q,al);

          if (r = vfs_go_find_one_in_list(al))
            {
		Obj_Delete(al);
              vfs_add_to_cache (type, fname);

              closedir (d);
              return (r);
            }
		Obj_Delete(al);
        }

    }

  closedir (d);

  return (NULL);
}


VFSAnswer_list vfs_answer_question (Question q, char *type)
{
  char buf[1024];
  DIR *d;
  struct dirent *ent;
  VFSAnswer_list al=CREATE_LIST(VFSAnswer);


  {
    char *f = vfs_getenv ("MGNVIEW");
    if (!f)
      return (al);
    strcpy (buf, f);
    strcat (buf, "/vfs/");
    strcat (buf, type);
  }

  d = opendir (buf);
  if (!d)
    {
      fprintf (MOAN, "vfs: no data for type %s\n", type);
      return (NULL);
    }

  while (ent = readdir (d))
    {
      char fname[1024], *r;
      strcpy (fname, buf);
      strcat (fname, "/");
      strcat (fname, ent->d_name);

      if (ent->d_name[0] != '.')
        {

          vfs_scan_vfs_file (fname, q,al);

        }

    }

  closedir (d);

  return (al);
}


/* This saves storing all of midrs... */
static char *
vfs_find_path_wrap (char *path, char *type)
{
  char chop[1024];
  char buf[VOL_MAX_LEN];
  char *p, *ret;
  Question q;

  q.qtype=VFS_Q_PATH;


  strcpy (buf, path);

  if (strcmp (type, "midr") == 0)
    {
      p = rindex (buf, '/');
      if (p)
        {
          *p = 0;
          strcat (buf, "/browse.img");
        }
    }


  q.qdata.path=buf;
  ret = vfs_one_answer_question (q, type);

  if (!ret)
    {
      return (ret);
    }

  if (strcmp (type, "midr") == 0)
    {
      strcpy (chop, ret);
      free (ret);

      p = rindex (chop, '/');
      if (!p)
        return (NULL);
      *p = 0;

      p = rindex (path, '/');
      if (!p)
        return (NULL);
      p++;

      strcat (chop, "/");
      strcat (chop, p);
      ret = strdup (chop);

    }


  return (ret);
}


int 
vfs_open (char *path, char *type, int flags)
{
  char *p;
  int fd = open (path, flags);
  if (fd > 0)
    return (fd);

  p = vfs_find_path_wrap (path, type);

  if (!p)
    return (fd);

  fd = open (p, flags);
  free (p);

  return (fd);
}

FILE *
vfs_fopen (char *path, char *type, char *mode)
{
  char *p;
  FILE *f = fopen (path, mode);
  if (f)
    return (f);

  p = vfs_find_path_wrap (path, type);

  if (!p)
    return (f);

  f = fopen (p, mode);
  free (p);

  return (f);
}

VFSAnswer_list vfs_find_covering(LL ll,char *type)
{
Question q;
VFSAnswer_list al;

q.qtype=VFS_Q_CONTAINS;
q.qdata.contains.ll=ll;

return(vfs_answer_question(q,type));
}

VFSAnswer_list vfs_find_path(char *path,char *type)
{
Question q;
VFSAnswer_list al;

q.qtype=VFS_Q_PATH;
q.qdata.path=path;

return(vfs_answer_question(q,type));
}

void vfs_print_answerlist(VFSAnswer_list al)
{
VFSAnswer a;

a=al->head;

if (!a) {
printf("cardinality of set of answers is zero\n");
return;
}
while(a)
{
printf("Medium:%s Volume:%s Path:%s\n",vfs_vol_type[a->v.type],a->v.id,a->path);
a=a->next;
}

}
